-----------------------------------------------------------------------------------------------------------------------------------
-- Entity Name:  BufrSubstitute
-- File Name:    BufrSubstitute.vhd
-- File Path:    \Projects\TigerSharc\Vhdl\FeaturedDesign\LinkPortReceiver
-- Project :
--
-- Purpose: 	Gray counter counting shifted bytes, generating CLKDIV.
--				Replacement for BUFR
--
-- Limitations: none
--
-- Authors: Marc Defossez
--
-- Tools: 
--
--  Contact:    e-mail  hotline@xilinx.com        phone   + 1 800 255 7778
--
--  Disclaimer:
--              LIMITED WARRANTY AND DISCLAIMER.
--              These designs are provided to you "as is". Xilinx and its licensors make and you
--              receive no warranties or conditions, express, implied, statutory or otherwise, and
--              Xilinx specifically disclaims any implied warranties of merchantability, non-infringement,
--              or fitness for a particular purpose. Xilinx does not warrant that the functions contained
--              in these designs will meet your requirements, or that the operation of these designs will
--              be uninterrupted or error free, or that defects in the designs will be corrected.
--              Furthermore, Xilinx does not warrant or make any representations regarding use or the
--              results of the use of the designs in terms of correctness, accuracy, reliability, or otherwise.
--
--              LIMITATION OF LIABILITY.
--              In no event will Xilinx or its licensors be liable for any loss of data, lost profits, cost or
--              procurement of substitute goods or services, or for any special, incidental, consequential,
--              or indirect damages arising from the use or operation of the designs or accompanying
--              documentation, however caused and on any theory of liability. This limitation will apply
--              even if Xilinx has been advised of the possibility of such damage. This limitation shall
--              apply not-withstanding the failure of the essential purpose of any limited remedies herein.
--
--              Copyright  2002 Xilinx, Inc.
--              All rights reserved
--
-- Revision History:       Created:       25/04/05
--                         Last opened:   Monday,Apr 25 05
--
------------------------------------------------------------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_UNSIGNED.all;
-- Simulation libraries
	library UNISIM;
	use UNISIM.all;
-- end simulation libraries

-- Libraries for Synplicity
-- Needed for components as LUT, SRL which get an INIT parameter.
-- When this library is not mentioned Synplify_Pro fails during synthesis of
-- the hardware with an error about the INIT value.
-- library virtex2;
-- use virtex2.components.all;
-- This library is ignored by Mentor Precision.
-- Only a warning is generated.
-- End libraries for Synplicity

entity BufrSubstitute is
    port (
        Clk : in std_logic;
        Ce : in std_logic;
        Clr : in std_logic;
        ClkDiv	: out std_logic
    );
end  BufrSubstitute ;

architecture BufrSubstitute_arch of BufrSubstitute  is
-----------------------------------------------------------------------------------------------------------------------------------
-- Component Instantiation
-----------------------------------------------------------------------------------------------------------------------------------
component FDCE		port (Q : out std_logic; D, C, CE, CLR : in std_logic); end component;
component FD		port (Q : out std_logic; D, C : in std_logic); end component;
component FD_1		port (Q : out std_logic; D, C : in std_logic); end component;
component LUT2
	generic (INIT : bit_vector (3 downto 0));
	port (I0, I1 : in std_logic; O : out std_logic); end component;
component LUT4
	generic (INIT : bit_vector (15 downto 0));
	port (I0, I1, I2, I3 : in std_logic; O : out std_logic); end component;
----------------------------------------------------------------------------------------------------------------------------------
-- Signal, Constants and Attributes Declarations
-----------------------------------------------------------------------------------------------------------------------------------
-- Constants
type InitIncVal2b is array (1 downto 0) of bit_vector (3 downto 0);
constant IncLutVal2b : InitIncVal2b := (
	X"A",	-- Address one
	X"3"	-- Address zero
	);
constant Tc2bMuxLutVal	: bit_vector (3 downto 0) := X"4"; -- IntLxClkInDiv Multiplexer.

-- Signals
signal IntByteCntOut	: std_logic_vector (1 downto 0);
signal IntByteIncOut 	: std_logic_vector (1 downto 0);

-- attributes
attribute RLOC : string;
	type RlocIncVal2b is array (1 downto 0) of string (4 downto 1);
	constant IncRlocVal2b : RlocIncVal2b := (
		"X0Y0",		-- Bit 1 (LUT and FF)
		"X0Y0"		-- Bit 0
	);
	attribute RLOC of ByteCntTcAndMuxLut 	: label is "X0Y1";
------------------------------------------------------------------------------------------------------------------------------------
--
Begin

-------------------------------------------------------------------
-- Byte counter, 2-Bit Gray counter.
-------------------------------------------------------------------
GenByteCnt : for i in 0 to 1 generate
	attribute RLOC of ByteIncrLut : label is IncRlocVal2b(i);
	attribute RLOC of ByteIncrFf : label is IncRlocVal2b(i);
	begin
	ByteIncrLut : LUT2
	generic map (
		INIT => IncLutVal2b(i)
	)
	port map (
		I0 	=> IntByteCntOut(0),
		I1 	=> IntByteCntOut(1),
		O 	=> IntByteIncOut(i)
	);
	ByteIncrFf : FD
	port map (
		D	=> IntByteIncOut(i),
		C	=> Clk,
		Q	=> IntByteCntOut(i)
	);
end generate;
-------------------------------------------------------------------
-- Byte counter terminal count and slow (CLKDIV) clock multiplexer.
-------------------------------------------------------------------
ByteCntTcAndMuxLut : LUT2
	generic map (
		INIT => Tc2bMuxLutVal
	)
	port map (
		I0 	=> IntByteCntOut(0),
		I1 	=> IntByteCntOut(1),
--		I2 	=> Ce,
--		I3 	=> Clr,
		O 	=> ClkDiv
	);
-------------------------------------------------------------------
end  BufrSubstitute_arch ;