-----------------------------------------------------------------------------------------------------------------------------------
-- Entity Name:  LinkPortRx
-- File Name:    LinkPortRx.vhd
-- File Path:    \Projects\TigerSharc\Vhdl\FeaturedDesign\LinkPortReceiver
-- Project :	ADSP-TS20x LinkPort
--
-- Purpose: 	Design an interface to/from the TigerSharc DSP in Virtex-4
--   			receiver part in the FPGA.
--
-- Limitations: none
--
-- Authors: Marc Defossez
--
-- Tools: 
--
--  Contact:    e-mail  hotline@xilinx.com        phone   + 1 800 255 2228
--
--  Disclaimer:
--              LIMITED WARRANTY AND DISCLAIMER.
--              These designs are provided to you "as is". Xilinx and its licensors make and you
--              receive no warranties or conditions, express, implied, statutory or otherwise, and
--              Xilinx specifically disclaims any implied warranties of merchantability, non-infringement,
--              or fitness for a particular purpose. Xilinx does not warrant that the functions contained
--              in these designs will meet your requirements, or that the operation of these designs will
--              be uninterrupted or error free, or that defects in the designs will be corrected.
--              Furthermore, Xilinx does not warrant or make any representations regarding use or the
--              results of the use of the designs in terms of correctness, accuracy, reliability, or otherwise.
--
--              LIMITATION OF LIABILITY.
--              In no event will Xilinx or its licensors be liable for any loss of data, lost profits, cost or
--              procurement of substitute goods or services, or for any special, incidental, consequential,
--              or indirect damages arising from the use or operation of the designs or accompanying
--              documentation, however caused and on any theory of liability. This limitation will apply
--              even if Xilinx has been advised of the possibility of such damage. This limitation shall
--              apply not-withstanding the failure of the essential purpose of any limited remedies herein.
--
--              Copyright  2002 Xilinx, Inc.
--              All rights reserved
--
-- Revision History:       Created:       21/02/05
--                         Last opened:   Monday,Feb 21 05
--
-----------------------------------------------------------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_UNSIGNED.all;
-- Simulation libraries
	-- synopsys translate_off
	library UNISIM;
	use UNISIM.all;
	-- synopsys translate_on
-- end simulation libraries

-- Libraries for Synplicity
-- Needed for components as LUT, SRL which get an INIT parameter.
-- When this library is not mentioned Synplify_Pro fails during synthesis of
-- the hardware with an error about the INIT value.
-- library virtex2;
-- use virtex2.components.all;
-- This library is ignored by Mentor Precision.
-- Only a warning is generated.
-- End libraries for Synplicity

entity LinkPortRx is
    port (
    	-- General
    	SysClk		: in std_logic;
    	SysRefClk	: in std_logic;
    	SystemRst	: in std_logic;		-- System reset used for rest of IDELAY_CTRL
    	SysRst		: in std_logic;		-- Reset of the logic after lock of DCM's and IDELAY_CTRL
    	SysLxAcko	: in std_logic;
    	SysRxFifoOe	: in std_logic;
    	SysRxEna	: in std_logic;
    	SysLxBcmpiN	: out std_logic;
    	SysRxIdlyCtrlRdy : out std_logic;
    	SysRxData	: out std_logic_vector (127 downto 0);
    	SysRxFifoStat : out std_logic_vector (3 downto 0);
		-- Link Port Receiver
		LxCLKINp		: in std_logic;
		LxCLKINn		: in std_logic;
		LxDATI_p		: in std_logic_vector (3 downto 0);
		LxDATI_n		: in std_logic_vector (3 downto 0);
		LxACKO			: out std_logic;
		LxBCMPIn		: in std_logic
		);
end  LinkPortRx ;

architecture LinkPortRx_arch of LinkPortRx  is
-----------------------------------------------------------------------------------------------------------------------------------
-- Component Instantiation
-----------------------------------------------------------------------------------------------------------------------------------
component IBUF_LVCMOS25		port (O : out std_logic; I : in  std_logic); end component;
component OBUF_LVCMOS25_S_12	port (O : out std_logic; I : in std_logic); end component;
component IBUFDS_LVDS_25	port (O : out std_logic; I, IB : in  std_logic); end component;
component OBUFDS_LVDS_25	port (I : in std_logic; O, OB : out  std_logic); end component;

component IBUFDS_DIFF_OUT	generic (IOSTANDARD	: string);
							port (I, IB	: in std_logic;		O, OB	: out std_logic); end component;
							
component BUFIO	port (O : out std_logic; I : in  std_logic); end component;
component BUFR	generic (BUFR_DIVIDE   : string);
				port (O : out std_logic; CE, CLR, I : in  std_logic); end component;
				
component BUFG	port (I : in std_logic;	O : out std_logic); end component;
component IBUFG	port (I : in std_logic;	O : out std_logic); end component;

component LUT4	generic (INIT : bit_vector);
				port (I0, I1, I2, I3 : in std_logic; O : out std_logic); end component;
				
component FDC		port (Q : out std_logic; D, C, CLR : in std_logic); end component;
	
component BufrSubstitute
    port (
        Clk		: in std_logic;
        Ce		: in std_logic;
        Clr 	: in std_logic;
        ClkDiv	: out std_logic
    );
end component;

component ISERDES
	generic (
	    BITSLIP_ENABLE	: boolean	:= false;
	    DATA_RATE		: string	:= "DDR";
	    DATA_WIDTH		: integer	:= 4;
	    INIT_Q1			: bit		:= '0';
	    INIT_Q2			: bit		:= '0';
	    INIT_Q3			: bit		:= '0';
	    INIT_Q4			: bit		:= '0';
	    INTERFACE_TYPE	: string	:= "MEMORY";
	    IOBDELAY		: string	:= "NONE";
	    IOBDELAY_TYPE	: string	:= "DEFAULT";
	    IOBDELAY_VALUE	: integer	:= 0;
	    NUM_CE			: integer	:= 2;
	    SERDES_MODE		: string	:= "MASTER";
	    SRVAL_Q1		: bit		:= '0';
	    SRVAL_Q2		: bit		:= '0';
	    SRVAL_Q3		: bit		:= '0';
	    SRVAL_Q4		: bit		:= '0'
	);
	port (
	    O			: out std_logic;
	    Q1			: out std_logic;
	    Q2			: out std_logic;
	    Q3			: out std_logic;
	    Q4			: out std_logic;
	    Q5			: out std_logic;
	    Q6			: out std_logic;
	    SHIFTOUT1	: out std_logic;
	    SHIFTOUT2	: out std_logic;
	    BITSLIP		: in std_logic;
	    CE1			: in std_logic;
	    CE2			: in std_logic;
	    CLK			: in std_logic;
	    CLKDIV		: in std_logic;
	    D			: in std_logic;
	    DLYCE		: in std_logic;
	    DLYINC		: in std_logic;
	    DLYRST		: in std_logic;
	    OCLK		: in std_logic;
	    REV			: in std_logic;
	    SHIFTIN1	: in std_logic;
	    SHIFTIN2	: in std_logic;
	    SR			: in std_logic
	);
end component;

component IDELAY
	generic	(
		IOBDELAY_TYPE : string;
		IOBDELAY_VALUE : integer
	);
	port(
		O 	: out std_logic;
		C 	: in std_logic;
		CE 	: in std_logic;
		I 	: in std_logic;
		INC : in std_logic;
		RST : in std_logic
	);
end component;

component IDELAYCTRL
	port (
		REFCLK	: in std_logic;
		RST		: in std_logic;
		RDY		: out std_logic
	);
end component;

component RxBramMem
    port (
    	-- BRAM port B = write port by the application
    	-- BRAM port A = read port by the transmitter
    	SysRst			: in std_logic;		-- System Reset
        SysClk			: in std_logic;		-- Application / system clock
        SysRxClk		: in std_logic;		-- Transmitter clock.
        SysRxRdEna		: in std_logic;		-- Read port enable.
        SysRxWrEna		: in std_logic;		-- Write port enable
        SysRxWrWen		: in std_logic_vector (3 downto 0);
        SysRxData		: out std_logic_vector (127 downto 0);
        SysRxDatIn		: in std_logic_vector (31 downto 0);
        SysRxRdAdd		: in std_logic_vector (8 downto 0);
        SysRxWrAdd		: in std_logic_vector (8 downto 0)
    );
end component;
-----------------------------------------------------------------------------------------------------------------------------------
-- Signal, Constants and Attributes Declarations
-----------------------------------------------------------------------------------------------------------------------------------
-- Constants
-- Signals
signal Low : std_logic;
signal High : std_logic;
-- System Signals
signal IntSysClkDiv	: std_logic;
-- Clocks
signal IntLxClkInP	: std_logic;	-- Net between IBUF_DIFF_OUT 'P' and BUFIO.I
signal IntLxClkInN	: std_logic;	-- Net between IBUF_DIFF_OUT 'N' and Byte/Quad Word Counter
signal IntLxClkInPidly	: std_Logic;
signal IntLxClkIn		: std_logic;	-- Fast IO clock
signal IntLxClkInNot	: std_logic;
signal IntLxClkInDiv	: std_logic;	-- Stobe to register the parallelized data in the ISERDES.

signal IntSysRefClk2Bufg : std_logic;
signal IntSysRefClk 	 : std_logic;
--
signal IntLxDati_p		: std_logic_vector (3 downto 0);	-- net between differential data inputs and ISERDES.D
signal IntLxDati_n		: std_logic_vector (3 downto 0);	-- net between differential data inputs and ISERDES.D
signal IntRxDat			: std_logic_vector (31 downto 0);	-- bus at output of ISERDES.

-- control signals
signal IntLxAcko		: std_logic := '1';
signal IntSysRxIdlyCtrlRdy	: std_logic;
signal IntSysRxIdlyCtrlRdyNot : std_logic;
signal IntLxBCMPIn		: std_logic;

signal IntRxShft_0		: std_logic;
signal IntRxShft		: std_logic_vector (3 downto 1);
signal IntRxWrWen		: std_logic_vector (3 downto 0);
signal IntRxWrAdd		: std_logic_vector (8 downto 0);
signal IntRxRdAdd		: std_logic_vector (8 downto 0);
signal IntRxWrAddFf		: std_logic_vector (8 downto 7);

-- Attributes
attribute INIT : string;
	attribute INIT of RxWenShft_0 : label is "1";	-- Set first bit of shift register to 1.
attribute LOC : string;
	attribute LOC of TheRefClk : label is "IDELAYCTRL_X0Y5"; -- 4VLX40-11FF1148
attribute RLOC_ORIGIN : string;
	attribute RLOC_ORIGIN of TheLxCnt : label is "X0Y176"; -- 4VLX40-11FF1148
	attribute RLOC_ORIGIN of TheRxBram : label is "X0Y20";
attribute KEEP : string;
	attribute KEEP of IntLxClkInN : signal is "TRUE";
	attribute KEEP of IntLxClkInDiv : signal is "TRUE";
	attribute KEEP of IntSysClkDiv : signal is "TRUE";
attribute MAXDELAY : string;
	attribute MAXDELAY of IntLxClkInN : signal is "800 ps";		-- 4VLX40-FF1148
	attribute MAXDELAY of IntSysClkDiv : signal is "500 ps";	-- 4VLX40-FF1148
attribute CLOCK_SIGNAL : string;
	attribute CLOCK_SIGNAL of IntLxClkInDiv : signal is "YES";
	attribute CLOCK_SIGNAL of IntSysClkDiv : signal is "YES";
	attribute CLOCK_SIGNAL of IntSysRxIdlyCtrlRdy : signal is "YES";
-----------------------------------------------------------------------------------------------------------------------------------
--
begin
---------------------------------------------------------------------------------------
-- Stuff
---------------------------------------------------------------------------------------
Low <= '0';
High <= '1';
---------------------------------------------------------------------------------------
-- Control Signals
---------------------------------------------------------------------------------------
TheLxAckoObuf	: OBUF_LVCMOS25_S_12 port map (I => IntLxAcko, O => LxACKO);	-- LxACKO receiver output.
IntLxAcko <= SysLxAcko;
SysLxBcmpiN <= IntLxBcmpiN;
TheLxBcmpiIbuf	: IBUF_LVCMOS25 port map (I => LxBcmpiN, O => IntLxBcmpiN);
---------------------------------------------------------------------------------------
-- IDELAY control for Voltage and temperature stabilisation.
-- Reference clock for the IDELAY_CTRL component used in the Receiver.
---------------------------------------------------------------------------------------
TheRefClkIbuf : IBUF_LVCMOS25	port map (I => SysRefClk, O => IntSysRefClk2Bufg);
TheRefClkBufg : BUFG	port map (I => IntSysRefClk2Bufg, O => IntSysRefClk);
TheRefClk : IDELAYCTRL	port map (REFCLK => IntSysRefClk, RST => SystemRst, RDY => IntSysRxIdlyCtrlRdy);
IntSysRxIdlyCtrlRdyNot <= not IntSysRxIdlyCtrlRdy;
SysRxIdlyCtrlRdy <= IntSysRxIdlyCtrlRdy;
---------------------------------------------------------------------------------------
-- Clock input from Link Port of DSP
---------------------------------------------------------------------------------------
-- Attributes for timing are written above.
TheClk : IBUFDS_DIFF_OUT
		generic map (
			IOSTANDARD => "LVDS_25"
		)
		port map (
			I => LxCLKINp,
			IB => LxCLKINn,
			O => IntLxClkInP,
			OB => IntLxClkInN		-- MAXDELAY attribute added
		);
TheClkDelay : IDELAY				-- IDELAY used as fixed delay. Pins to control IDELAY must be 
	generic	map (					-- tied to GND.
		IOBDELAY_TYPE => "FIXED",
		IOBDELAY_VALUE => 0
	)
	port map (
		O 	=> IntLxClkInPidly,
		I 	=> IntLxClkInP,
		C 	=> Low,
		CE 	=> low,
		INC => Low,
		RST => Low
	);
TheBufIo : BUFIO port map (I => IntLxClkInPidly, O => IntLxClkIn);		-- Fast clock for the ISERDES components
IntLxClkInNot <= not IntLxClkIn;
---------------------------------------------------------------------------------------
-- Gray Counter as BUFR replacement.
TheLxCnt : BufrSubstitute				-- This counter takes the function of the BUFR.
    port map (
        Clk 	=> IntLxClkInN,
        Ce		=> High,				-- Select terminal count at "00"
        Clr 	=> Low,
        ClkDiv	=> IntLxClkInDiv		-- Divided clock, MAXDELAY attribute added.
    );
-- IntLxClkInDiv is ISERDES.CLKDIV clock and one branch of the signal goes to the BUFR.I.
---------------------------------------------------------------------------------------
-- BUFR put after BUFR substitute Gray-counter.
--
-- BUFR used in bypass mode to distribute the clock locally.
-- BUFR in bypass has a delay of 420ps and the shortest connection
-- between a SLICE and the input of the BUFR (in a 4VLX40) is 1.3 ns.
-- The total delay from a FF to the clock input of another FF or BRAM is 
--         Tcko + net + Tbrcko_BYP + net = 0.291 + 1.291 + 0.420 + 0.600 = 2.6 ns
-- This clock delay is no problem since the BUFR-substite-counter divides the clock of 
-- ~500 MHz in 3.5. Thus the period becomes ~ 6ns.
--
TheBufr : BUFR
	generic map (BUFR_DIVIDE => "BYPASS")
	port map  (
		I => IntLxClkInDiv,
		CE => Low,
		CLR => Low,
		O => IntSysClkDiv
	);
-- IntSysClkDiv <= IntLxClkInDiv;
---------------------------------------------------------------------------------------	
-- The data inputs of Link Port from DSP.
-- Data input using a differential input, differential output buffer and 
-- two ISERDES elements. One clocked on rising edge and the other clocked 
-- on the falling edge creating an odd and an even parallel word that need merging
-- for completness.
--
-- A generic parameter is used to instantiate the correct number of IBUFDS, ISERDES
-- combinations.
-- The output is a large bus, combining all outputs of all ISERDES elements.
-- This bus must be split for further use in the design.
------------------------------------------------------------------------------------
TheLxDati : for n in 3 downto 0 generate
	DatIn : IBUFDS_DIFF_OUT
		generic map (
			IOSTANDARD => "LVDS_25"
		)
		port map (
			I => LxDATI_p(n),
			IB => LxDATI_n(n),
			O => IntLxDati_p(n),
			OB => IntLxDati_n(n)
		);
	LxDatiPserdes : ISERDES
		generic map (
		    BITSLIP_ENABLE	=> FALSE,
		    DATA_RATE		=> "SDR",
		    DATA_WIDTH		=> 4,
		    INIT_Q1			=> '0',
		    INIT_Q2			=> '0',
		    INIT_Q3			=> '0',
		    INIT_Q4			=> '0',
		    INTERFACE_TYPE	=> "MEMORY",
		    IOBDELAY		=> "IFD",			-- <<<<
		    IOBDELAY_TYPE	=> "FIXED",
		    IOBDELAY_VALUE	=> 9,				-- <<<<
		    NUM_CE			=> 1,
		    SERDES_MODE		=> "MASTER",
		    SRVAL_Q1		=> '0',
		    SRVAL_Q2		=> '0',
		    SRVAL_Q3		=> '0',
		    SRVAL_Q4		=> '0'
		    )
		port map (
		    O			=> open,
		    Q1			=> IntRxDat(8*(n+1)-2),
		    Q2			=> IntRxDat(8*(n+1)-4),
		    Q3			=> IntRxDat(8*(n+1)-6),
		    Q4			=> IntRxDat(8*(n+1)-8),
		    Q5			=> open,
		    Q6			=> open,
		    SHIFTOUT1	=> open,
		    SHIFTOUT2	=> open,
		    BITSLIP		=> Low,
		    CE1			=> High,
		    CE2			=> High,
		    CLK			=> IntLxClkIn,
		    CLKDIV		=> IntLxClkInDiv,
		    D			=> IntLxDati_p(n),
		    DLYCE		=> Low,
		    DLYINC		=> Low,
		    DLYRST		=> IntSysRxIdlyCtrlRdyNot,
		    OCLK		=> Low,
		    REV			=> Low,
		    SHIFTIN1	=> Low,
		    SHIFTIN2	=> Low,
		    SR			=> SysRst
		  );
	LxDatiNserdes : ISERDES
		generic map (
		    BITSLIP_ENABLE	=> FALSE,
		    DATA_RATE		=> "SDR",
		    DATA_WIDTH		=> 4,
		    INIT_Q1			=> '0',
		    INIT_Q2			=> '0',
		    INIT_Q3			=> '0',
		    INIT_Q4			=> '0',
		    INTERFACE_TYPE	=> "MEMORY",
		    IOBDELAY		=> "IFD", --"NONE",
		    IOBDELAY_TYPE	=> "FIXED",
		    IOBDELAY_VALUE	=> 9,
		    NUM_CE			=> 1,
		    SERDES_MODE		=> "MASTER",
		    SRVAL_Q1		=> '0',
		    SRVAL_Q2		=> '0',
		    SRVAL_Q3		=> '0',
		    SRVAL_Q4		=> '0'
		    )
		port map (
		    O			=> open,
		    Q1			=> IntRxDat(8*(n+1)-1),
		    Q2			=> IntRxDat(8*(n+1)-3),
		    Q3			=> IntRxDat(8*(n+1)-5),
		    Q4			=> IntRxDat(8*(n+1)-7),
		    Q5			=> open,
		    Q6			=> open,
		    SHIFTOUT1	=> open,
		    SHIFTOUT2	=> open,
		    BITSLIP		=> Low,
		    CE1			=> High,
		    CE2			=> High,
		    CLK			=> IntLxClkInNot,
		    CLKDIV		=> IntLxClkInDiv,
		    D			=> IntLxDati_n(n),
		    DLYCE		=> Low,
		    DLYINC		=> Low,
		    DLYRST		=> IntSysRxIdlyCtrlRdyNot,
		    OCLK		=> Low,
		    REV			=> Low,
		    SHIFTIN1	=> Low,
		    SHIFTIN2	=> Low,
		    SR			=> SysRst
		  );
end generate;
---------------------------------------------------------------------------------------
-- ISERDES parallel output.
---------------------------------------------------------------------------------------
-- Data arrangement is:
-- 	Bit 	Pos		ISERDES.Q	Bit #	Stream #
--	Lane	Neg		output		bus		bit	
------------------------------------------------------
--	0		P		1			30			24
--					2			28			16
--					3			26			8
--					4			24			0
--			N		1			31			28
--					2			29			20
--					3			27			12
--					4			25			4
------------------------------------------------------
--	1		P		1			22			25
--					2			20			17
--					3			18			9
--					4			16			1
--			N		1			23			29
--					2			21			21
--					3			19			13
--					4			17			5
------------------------------------------------------
--	2		P		1			14			26
--					2			12			18
--					3			10			10
--					4			8			2
--			N		1			15			30
--					2			13			22
--					3			11			14
--					4			9			6
------------------------------------------------------
--	3		P		1			6			27
--					2			4			19
--					3			2			11
--					4			0			3
--			N		1			7			31
--					2			5			23
--					3			3			15
--					4			1			7
---------------------------------------------------------------------------------------
-- ISERDES parallel output to Block RAM input. 
---------------------------------------------------------------------------------------
TheRxBram : RxBramMem
    port map (
    	-- BRAM port B = write port by the application
    	-- BRAM port A = read port by the transmitter
    	SysRst			=> SysRst,
        SysClk			=> SysClk,
        SysRxClk		=> IntSysClkDiv,
        SysRxRdEna		=> SysRxFifoOe,
        SysRxWrEna		=> SysRxEna,
        SysRxWrWen		=> IntRxWrWen,
        SysRxData		=> SysRxData,
        SysRxDatIn		=> IntRxDat,
        SysRxRdAdd		=> IntRxRdAdd,
        SysRxWrAdd		=> IntRxWrAdd
    );
---------------------------------------------------------------------------------------
-- Block RAM address counters. 
---------------------------------------------------------------------------------------
RxWenShft_0 : process (SysRst, IntSysClkDiv, SysRxEna)
begin
	if (SysRst = '1') then
		IntRxShft_0 <= '1';
	elsif (IntSysClkDiv'event and IntSysClkDiv = '1') then
		if (SysRxEna = '1') then
			IntRxShft_0 <= IntRxWrWen(3);
		end if;
	end if;
end process;

RxWenShft : process (SysRst, IntSysClkDiv, SysRxEna)
begin
	if (SysRst = '1') then
		IntRxShft <= "000";
	elsif (IntSysClkDiv'event and IntSysClkDiv = '1') then
		if (SysRxEna = '1') then
			IntRxShft(1) <= IntRxShft_0;
			IntRxShft(2) <= IntRxShft(1);
			IntRxShft(3) <= IntRxShft(2);
		end if;
	end if;
end process;

IntRxWrWen <= IntRxShft (3 downto 1) & IntRxShft_0;
--
RxBramWrAddrGen : process (IntSysClkDiv, SysRst, SysRxEna)
begin
if (SysRst = '1') then
	IntRxWrAdd <= (others => '0');
elsif (IntSysClkDiv'event and IntSysClkDiv = '1') then
	if (SysRxEna = '1') then
		IntRxWrAdd(2 downto 0) <= IntRxWrAdd(2 downto 0) + "01"; -- 3-bit Binary counter 
		if (IntRxWrAdd(2 downto 0) = "111") then
			case IntRxWrAdd(5 downto 3) is
				when "000" => IntRxWrAdd(5 downto 3) <= "001";
				when "001" => IntRxWrAdd(5 downto 3) <= "011";
				when "011" => IntRxWrAdd(5 downto 3) <= "010";
				when "010" => IntRxWrAdd(5 downto 3) <= "110";
				when "110" => IntRxWrAdd(5 downto 3) <= "111";
				when "111" => IntRxWrAdd(5 downto 3) <= "101";
				when "101" => IntRxWrAdd(5 downto 3) <= "100";
				when others => IntRxWrAdd(5 downto 3) <= "000";
			end case;
		end if;
		if (IntRxWrAdd(5 downto 3) = "100" and IntRxWrAdd(2 downto 0) = "111") then
			case IntRxWrAdd(8 downto 6) is
				when "000" => IntRxWrAdd(8 downto 6) <= "001";
				when "001" => IntRxWrAdd(8 downto 6) <= "011";
				when "011" => IntRxWrAdd(8 downto 6) <= "010";
				when "010" => IntRxWrAdd(8 downto 6) <= "110";
				when "110" => IntRxWrAdd(8 downto 6) <= "111";
				when "111" => IntRxWrAdd(8 downto 6) <= "101";
				when "101" => IntRxWrAdd(8 downto 6) <= "100";
				when others => IntRxWrAdd(8 downto 6) <= "000";
			end case;
		end if;
	end if;
end if;
end process;

RxBramRdAddrGen : process (SysClk, SysRst, SysRxFifoOe)
begin
if (SysRst = '1') then
	IntRxRdAdd <= (others => '0');
elsif (SysClk'event and SysClk = '1') then
	if (SysRxFifoOe = '1') then
		IntRxRdAdd(2 downto 0) <= IntRxRdAdd(2 downto 0) + "01"; -- 3-bit Binary counter 
		if (IntRxRdAdd(2 downto 0) = "111") then
			case IntRxRdAdd(5 downto 3) is
				when "000" => IntRxRdAdd(5 downto 3) <= "001";
				when "001" => IntRxRdAdd(5 downto 3) <= "011";
				when "011" => IntRxRdAdd(5 downto 3) <= "010";
				when "010" => IntRxRdAdd(5 downto 3) <= "110";
				when "110" => IntRxRdAdd(5 downto 3) <= "111";
				when "111" => IntRxRdAdd(5 downto 3) <= "101";
				when "101" => IntRxRdAdd(5 downto 3) <= "100";
				when others => IntRxRdAdd(5 downto 3) <= "000";
			end case;
		end if;
		if (IntRxRdAdd(5 downto 3) = "100" and IntRxRdAdd(2 downto 0) = "111") then
			case IntRxRdAdd(8 downto 6) is
				when "000" => IntRxRdAdd(8 downto 6) <= "001";
				when "001" => IntRxRdAdd(8 downto 6) <= "011";
				when "011" => IntRxRdAdd(8 downto 6) <= "010";
				when "010" => IntRxRdAdd(8 downto 6) <= "110";
				when "110" => IntRxRdAdd(8 downto 6) <= "111";
				when "111" => IntRxRdAdd(8 downto 6) <= "101";
				when "101" => IntRxRdAdd(8 downto 6) <= "100";
				when others => IntRxRdAdd(8 downto 6) <= "000";
			end case;
		end if;
	end if;
end if;
end process;

GenOne : for n in 7 to 8 generate
	MsbRdAddrFf : FDC
		port map (
			D	=> IntRxWrAdd(n),
			C	=> SysClk,
			CLR	=> SysRst,
			Q	=> IntRxWrAddFf(n)
			);
end generate;

RxFifoFlags : process (SysClk, SysRst, IntRxWrAddFf(8 downto 7), IntRxRdAdd(8 downto 7))
subtype RxRdWrAddr is std_logic_vector (3 downto 0);
begin
	if (SysRst = '1') then
		SysRxFifoStat <= (others => '0');
	elsif (SysClk'event and SysClk = '1') then
		case RxRdWrAddr'(IntRxWrAddFf(8 downto 7) & IntRxRdAdd(8 downto 7)) is
			--   Write-Read								-- Between 3/4 and full
			when "0001" => SysRxFifoStat(3) <= '1';
			when "0111" => SysRxFifoStat(3) <= '1';
			when "1110" => SysRxFifoStat(3) <= '1';
			when "1000" => SysRxFifoStat(3) <= '1';
			when others => SysRxFifoStat(3) <= '0';
		end case;
		case RxRdWrAddr'(IntRxWrAddFf(8 downto 7) & IntRxRdAdd(8 downto 7)) is
			--   Write-Read								-- Between half and 3/4
			when "0011" => SysRxFifoStat(2) <= '1';
			when "0110" => SysRxFifoStat(2) <= '1';
			when "1100" => SysRxFifoStat(2) <= '1';
			when "1001" => SysRxFifoStat(2) <= '1';
			when others => SysRxFifoStat(2) <= '0';
		end case;
		case RxRdWrAddr'(IntRxWrAddFf(8 downto 7) & IntRxRdAdd(8 downto 7)) is
			--   Write-Read								-- Between 1/4 and half
			when "0010" => SysRxFifoStat(1) <= '1';
			when "0100" => SysRxFifoStat(1) <= '1';
			when "1101" => SysRxFifoStat(1) <= '1';
			when "1011" => SysRxFifoStat(1) <= '1';
			when others => SysRxFifoStat(1) <= '0';
		end case;
		case RxRdWrAddr'(IntRxWrAddFf(8 downto 7) & IntRxRdAdd(8 downto 7)) is
			--   Write-Read								-- Between Empty and 1/4
			when "0000" => SysRxFifoStat(0) <= '1';
			when "0101" => SysRxFifoStat(0) <= '1';
			when "1111" => SysRxFifoStat(0) <= '1';
			when "1010" => SysRxFifoStat(0) <= '1';
			when others => SysRxFifoStat(0) <= '0';
		end case;
	end if;
end process;
--
---------------------------------------------------------------------------------------
end  LinkPortRx_arch ;