
------------------------------------------------------------------------------------------------------------------------
-- Copyright (c) 1996-2003 Xilinx, Inc.
-- All Rights Reserved
------------------------------------------------------------------------------------------------------------------------
--
--  Disclaimer:
--              LIMITED WARRANTY AND DISCLAIMER.
--              These designs are provided to you "as is". Xilinx and its licensors make and you
--              receive no warranties or conditions, express, implied, statutory or otherwise, and
--              Xilinx specifically disclaims any implied warranties of merchantability, non-infringement,
--              or fitness for a particular purpose. Xilinx does not warrant that the functions contained
--              in these designs will meet your requirements, or that the operation of these designs will
--              be uninterrupted or error free, or that defects in the designs will be corrected.
--              Furthermore, Xilinx does not warrant or make any representations regarding use or the
--              results of the use of the designs in terms of correctness, accuracy, reliability, or otherwise.
--
--              LIMITATION OF LIABILITY.
--              In no event will Xilinx or its licensors be liable for any loss of data, lost profits, cost or
--              procurement of substitute goods or services, or for any special, incidental, consequential,
--              or indirect damages arising from the use or operation of the designs or accompanying
--              documentation, however caused and on any theory of liability. This limitation will apply
--              even if Xilinx has been advised of the possibility of such damage. This limitation shall
--              apply not-withstanding the failure of the essential purpose of any limited remedies herein.
--
--  Contact:    e-mail  hotline@xilinx.com        phone   + 1 800 255 7778
--   ____  ____
--  /   /\/   /
-- /___/  \  /   Vendor: Xilinx
-- \   \   \/    Version: 
--  \   \        Filename: LinkPortTx.vhd
--  /   /        Date Last Modified:  
-- /___/   /\    Date Created: 22/06/05
-- \   \  /  \
--  \___\/\___\
-- 
-- Device: 		XC4LX40-11FF1148
-- Author:		marc Defossez
-- Entity Name: LinkPortTx
-- Purpose:  	Design an interface to/from the TigerSharc DSP in Virtex-4.
--   			Transmitter part in the FPGA.
-- Tools:		ISE7.1.02
-- Limitations: none
--
-- Revision History:
--    Rev. 
--
------------------------------------------------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_UNSIGNED.all;
-- Simulation libraries
	-- synopsys translate_off
	library UNISIM;
	use UNISIM.all;
	-- synopsys translate_on
-- end simulation libraries

-- Libraries for Synplicity
-- Needed for components as LUT, SRL which get an INIT parameter.
-- When this library is not mentioned Synplify_Pro fails during synthesis of
-- the hardware with an error about the INIT value.
-- library virtex2;
-- use virtex2.components.all;
-- This library is ignored by Mentor Precision.
-- Only a warning is generated.
-- End libraries for Synplicity

entity LinkPortTx is
    port (
    	-- general
        SysTxClkP	: in std_logic;		-- Clock to DCM for transmitter.
        SysTxClkN	: in std_logic;
        SysClk		: in std_logic;		-- System clock from a system DCM
        SystemRst	: in std_logic;		-- System Reset
        SysRst		: in std_logic;		-- Reset after DCM's have been locked.
        SysTxDcmLckd : out std_logic;	-- locked condition of the transmitter DCM.
        SysTxFifoEna : in std_logic;		-- BRAM Memory enable from application side.
        SysTxEna	: in std_logic;		-- Transmitter enable.
        SysLxBcmpoN	: in std_logic;		-- LxBCMPIn passed from back-end design to transmitter.
        SysLxAcki	: out std_logic;	-- LxACKI signal passed through to the application design.
        SysTxData	: in std_logic_vector (127 downto 0);
        SysTxFifoStat : out std_logic_vector (7 downto 0); -- Fine grained status FIFO status bits.
        -- Link Port Transmitter
        LxCLKOUTp	: out std_logic;
        LxCLKOUTn	: out std_logic;
        LxDATO_p	: out std_logic_vector (3 downto 0);
        LxDATO_n	: out std_logic_vector (3 downto 0);
        LxACKI		: in std_logic;
        LxBCMPOn	: out std_logic
    );
end  LinkPortTx ;

architecture LinkPortTx_arch of LinkPortTx  is
------------------------------------------------------------------------------------------------------------------------------------
-- Component Instantiation
------------------------------------------------------------------------------------------------------------------------------------
component IBUFGDS	generic (DIFF_TERM : boolean := FALSE;
							 IOSTANDARD : string := "DEFAULT");
					port (I, IB : in std_logic; O  : out std_logic);	end component;
component BUFG		port (I : in std_logic; O : out std_logic);	end component;
component IBUF_LVCMOS25			port (O : out std_logic; I : in  std_logic); end component;
component OBUF_LVCMOS25_S_12	port (O : out std_logic; I : in std_logic); end component;
component OBUFDS_LVDS_25		port (O, OB : out std_logic; I : in std_logic); end component;
component FDC		port (Q : out std_logic; D, C, CLR : in std_logic); end component;

component DCM_ADV
   generic (
		CLK_FEEDBACK 			: string :=  "1X";
		CLKDV_DIVIDE 			: real :=  2.000000;
		CLKFX_DIVIDE 			: integer :=  1;
		CLKFX_MULTIPLY 			: integer :=  4;
		CLKIN_DIVIDE_BY_2 		: boolean :=  FALSE;
		CLKIN_PERIOD 			: real :=  0.000000;
		CLKOUT_PHASE_SHIFT 		: string :=  "NONE";
		DCM_PERFORMANCE_MODE 	: string :=  "MAX_SPEED";
		DESKEW_ADJUST 			: string :=  "SYSTEM_SYNCHRONOUS";
		DFS_FREQUENCY_MODE 		: string :=  "LOW";
		DLL_FREQUENCY_MODE 		: string :=  "LOW";
		DUTY_CYCLE_CORRECTION 	: boolean :=  TRUE;
		PHASE_SHIFT 			: integer :=  0;
		STARTUP_WAIT 			: boolean :=  FALSE;
		FACTORY_JF 				: bit_vector :=  x"F0F0"
    );
	port (
		CLKIN    : in    std_logic; 
		CLKFB    : in    std_logic; 
		DADDR    : in    std_logic_vector (6 downto 0); 
		DI       : in    std_logic_vector (15 downto 0); 
		DWE      : in    std_logic; 
		DEN      : in    std_logic; 
		DCLK     : in    std_logic; 
		RST      : in    std_logic; 
		PSEN     : in    std_logic; 
		PSINCDEC : in    std_logic; 
		PSCLK    : in    std_logic; 
		CLK0     : out   std_logic; 
		CLK90    : out   std_logic; 
		CLK180   : out   std_logic; 
		CLK270   : out   std_logic; 
		CLKDV    : out   std_logic; 
		CLK2X    : out   std_logic; 
		CLK2X180 : out   std_logic; 
		CLKFX    : out   std_logic; 
		CLKFX180 : out   std_logic; 
		DRDY     : out   std_logic; 
		DO       : out   std_logic_vector (15 downto 0); 
		LOCKED   : out   std_logic; 
		PSDONE   : out   std_logic
    );
end component;

component OSERDES
	generic (
		DATA_RATE_OQ	: string := "DDR";
		DATA_RATE_TQ	: string := "DDR";
		DATA_WIDTH		: integer := 4;
		INIT_OQ			: bit := '0';
		INIT_TQ			: bit := '0';
		SERDES_MODE		: string := "MASTER";
		SRVAL_OQ		: bit := '0';
		SRVAL_TQ		: bit := '0';
		TRISTATE_WIDTH	: integer := 4
	);
	port (
		OQ			: out std_logic;
		SHIFTOUT1	: out std_logic;
		SHIFTOUT2	: out std_logic;
		TQ			: out std_logic;
		CLK			: in std_logic;
		CLKDIV		: in std_logic;
		D1 			: in std_logic;
		D2 			: in std_logic;
		D3 			: in std_logic;
		D4 			: in std_logic;
		D5 			: in std_logic;
		D6 			: in std_logic;
		OCE 		: in std_logic;
		REV 		: in std_logic;
		SHIFTIN1 	: in std_logic;
		SHIFTIN2 	: in std_logic;
		SR 			: in std_logic;
		T1 			: in std_logic;
		T2 			: in std_logic;
		T3 			: in std_logic;
		T4 			: in std_logic;
		TCE 		: in std_logic
	);
end component;

component TxBramMem
    port (
    	-- BRAM port B = write port by the application
    	-- BRAM port A = read port by the transmitter
    	SysRst			: in std_logic;		-- System Reset
        SysClk			: in std_logic;		-- Application / system clock
        SysTxClk		: in std_logic;		-- Transmitter clock.
        SysTxRdEna		: in std_logic;		-- Read port enable.
        SysTxWrEna		: in std_logic;		-- Write port enable
        SysTxData		: in std_logic_vector (127 downto 0);
        SysTxDatOut		: out std_logic_vector (15 downto 0);
        SysTxRdAdd		: in std_logic_vector (11 downto 0);
        SysTxWrAdd		: in std_logic_vector (8 downto 0)
    );
end component;
------------------------------------------------------------------------------------------------------------------------------------
-- Signal, Constants and Attributes Declarations
------------------------------------------------------------------------------------------------------------------------------------
-- Constants

-- Signals
signal IntIbufgToDcm	: std_logic;
signal IntClk0ToBufg	: std_logic;
signal IntClk90ToBufg	: std_logic;
signal IntClk0			: std_logic;
signal IntClk90			: std_logic;
signal IntDcmLocked		: std_logic;

signal IntLxAcki		: std_logic;
signal IntSysLxBcmpoN	: std_logic;

signal IntTxDat			: std_logic_vector (15 downto 0);
signal IntLxDato		: std_logic_vector (3 downto 0);
signal IntLxClkOut		: std_logic;

signal IntTxWrAdd		: std_logic_vector (8 downto 0);
signal IntTxRdAdd		: std_logic_vector (11 downto 0);
signal IntTxRdAddFf		: std_logic_vector (11 downto 9);
signal IntTxRdEna		: std_logic;

signal IntProceed_A		: std_logic;
signal IntProceedNot_A	: std_logic;
signal IntProceed_B		: std_logic;
signal IntProceedNot_B	: std_logic;

signal IntProceed90		: std_logic;
signal IntProceed90Not	: std_logic;


signal LowSixTeen		: std_logic_vector (15 downto 0);
signal LowSeven			: std_logic_vector (6 downto 0);
signal Low              : std_logic;
signal High				: std_logic;

-- Attributes
attribute RLOC_ORIGIN : string;
	attribute RLOC_ORIGIN of TheTxBram : label is "X0Y16";	-- 4VLX40-11FF1148
attribute KEEP : string;
	attribute KEEP of IntProceed_A : signal is "TRUE";
	attribute KEEP of IntProceed_B : signal is "TRUE";
attribute MAXDELAY : string;
	attribute MAXDELAY of IntProceed_A : signal is "1000 ps";	-- 4VLX40-FF1148
	attribute MAXDELAY of IntProceed_B : signal is "900 ps";	-- 4VLX40-FF1148
attribute MAXSKEW : string;
	attribute MAXSKEW of IntProceed_A : signal is "300 ps";	-- 4VLX40-FF1148
	attribute MAXSKEW of IntProceed_B : signal is "300 ps";	-- 4VLX40-FF1148
-----------------------------------------------------------------------------------------------------------------------------------
--
begin
---------------------------------------------------------------------------------------
-- Stuff
---------------------------------------------------------------------------------------
Low <= '0';
LowSixTeen <= X"0000";
LowSeven <= "0000000";
High <= '1';
---------------------------------------------------------------------------------------
-- Inputs and Outputs.
-- Non dedicated inputs and outputs are mentionned here.
---------------------------------------------------------------------------------------
-- Control signals inputs and outputs. LxACKI and LxBCMPOn.
TheLxAckiBuf : IBUF_LVCMOS25	port map (I => LxACKI, O => IntLxAcki);
SysLxAcki <= IntLxAcki;

IntSysLxBcmpoN <= SyslxBcmpoN;
TheLxBcmpoNBuf : OBUF_LVCMOS25_S_12	port map (I => IntSysLxBcmpoN, O => LxBCMPOn);
---------------------------------------------------------------------------------------
-- Transmitter clocking
---------------------------------------------------------------------------------------
TheLxTxIbufgds : IBUFGDS
	generic map (
		DIFF_TERM	=>	FALSE,
		IOSTANDARD	=>	"LVDS_25"
	)
	port map (
		I	=> SysTxClkP,
		IB	=> SysTxClkN,
		O	=> IntIbufgToDcm
	);
             
TheLxTxDcm : DCM_ADV
	generic map (
		CLK_FEEDBACK 		=> "1X",
		CLKDV_DIVIDE 		=> 4.000000,
		CLKFX_DIVIDE 		=> 1,
		CLKFX_MULTIPLY 		=> 4,
		CLKIN_DIVIDE_BY_2 	=> FALSE,
		CLKIN_PERIOD 		=> 2.222220,
		CLKOUT_PHASE_SHIFT 	=> "NONE",
		DCM_PERFORMANCE_MODE =>	"MAX_SPEED",
		DESKEW_ADJUST 		=> "SYSTEM_SYNCHRONOUS",
		DFS_FREQUENCY_MODE 	=> "HIGH",
		DLL_FREQUENCY_MODE	=> "HIGH",
		DUTY_CYCLE_CORRECTION => TRUE,
		PHASE_SHIFT 		=> 0,
		STARTUP_WAIT 		=>	FALSE,
		FACTORY_JF 			=> X"F0F0"
	)
	port map (
   		CLKFB		=> IntClk0,
		CLKIN		=> IntIbufgToDcm,
		DADDR		=> LowSeven,
		DCLK		=> Low,
		DEN			=> Low,
		DI			=> LowSixTeen,
		DWE			=> Low,
		PSCLK		=> Low,
		PSEN		=> Low,
		PSINCDEC	=> Low,
		RST			=> SystemRst,
		CLKDV		=> open,
		CLKFX		=> open,
		CLKFX180	=> open,
		CLK0		=> IntClk0ToBufg,
		CLK2X		=> open,
		CLK2X180	=> open,
		CLK90		=> IntClk90ToBufg,
		CLK180		=> open,
		CLK270		=> open,
		DO			=> open,
		DRDY		=> open,
		LOCKED		=> IntDcmLocked,
		PSDONE		=> open
	);
TheClk0Bufg : BUFG	port map (I => IntClk0ToBufg, O => IntClk0);	-- Clock for the DATA OSERDES.
TheClk90Bufg : BUFG	port map (I => IntClk90ToBufg, O => IntClk90);	-- Clock for the CLOCK OSERDES.
SysTxDcmLckd <= IntdcmLocked;
---------------------------------------------------------------------------------------
-- IntClk0 Proceed signal
IntProceedNot_A <= not IntProceed_A;
TheClkPrcd_A : FDC 
	port map (
	D	=> IntProceedNot_A,
	C	=> IntClk0,
	CLR => SysRst,
	Q	=> IntProceed_A
	);
IntProceedNot_B <= not IntProceed_B;
TheClkPrcd_B : FDC 
	port map (
	D	=> IntProceedNot_B,
	C	=> IntClk0,
	CLR => SysRst,
	Q	=> IntProceed_B
	);

-- IntClk90 Proceed signal
IntProceed90Not <= not IntProceed90;
TheClk90Prcd : FDC
	port map (
	D	=> IntProceed90Not,
	C	=> IntClk90,
	CLR => SysRst,
	Q	=> IntProceed90
	);
---------------------------------------------------------------------------------------
-- BRAM FIFO and memory
---------------------------------------------------------------------------------------
IntTxRdEna <= not IntProceed_A;
TheTxBram : TxBramMem
    port map (
    	-- BRAM port B = write port by the application
    	-- BRAM port A = read port by the transmitter
    	SysRst			=> SysRst,
        SysClk			=> SysClk,
        SysTxClk		=> IntCLk0,
        SysTxRdEna		=> IntTxRdEna,
        SysTxWrEna		=> SysTxFifoEna,
        SysTxData		=> SysTxData,
        SysTxDatOut		=> IntTxDat,
        SysTxRdAdd		=> IntTxRdAdd,	-- The transmitter read from the BRAM
        SysTxWrAdd		=> IntTxWrAdd		-- The systems writes into the BRAM
    );

TxBramWrAddrGen : process (SysClk, SysRst, SysTxFifoEna)
begin
if (SysRst = '1') then
	IntTxWrAdd <= (others => '0');
elsif (SysClk'event and SysClk = '1') then
	if (SysTxFifoEna = '1') then
		IntTxWrAdd(2 downto 0) <= IntTxWrAdd(2 downto 0) + "01"; -- 3-bit Binary counter 
		if (IntTxWrAdd(2 downto 0) = "111") then
			case IntTxWrAdd(5 downto 3) is
				when "000" => IntTxWrAdd(5 downto 3) <= "001";
				when "001" => IntTxWrAdd(5 downto 3) <= "011";
				when "011" => IntTxWrAdd(5 downto 3) <= "010";
				when "010" => IntTxWrAdd(5 downto 3) <= "110";
				when "110" => IntTxWrAdd(5 downto 3) <= "111";
				when "111" => IntTxWrAdd(5 downto 3) <= "101";
				when "101" => IntTxWrAdd(5 downto 3) <= "100";
				when others => IntTxWrAdd(5 downto 3) <= "000";
			end case;
		end if;
		if (IntTxWrAdd(5 downto 3) = "100" and IntTxWrAdd(2 downto 0) = "111") then
			case IntTxWrAdd(8 downto 6) is
				when "000" => IntTxWrAdd(8 downto 6) <= "001";
				when "001" => IntTxWrAdd(8 downto 6) <= "011";
				when "011" => IntTxWrAdd(8 downto 6) <= "010";
				when "010" => IntTxWrAdd(8 downto 6) <= "110";
				when "110" => IntTxWrAdd(8 downto 6) <= "111";
				when "111" => IntTxWrAdd(8 downto 6) <= "101";
				when "101" => IntTxWrAdd(8 downto 6) <= "100";
				when others => IntTxWrAdd(8 downto 6) <= "000";
			end case;
		end if;
	end if;
end if;
end process;

TxBramRdAddrGen : process (IntClk0, SysRst, SysTxEna, IntProceed_A)
begin
if (SysRst = '1') then
	IntTxRdAdd <= (others => '0');
elsif (IntClk0'event and IntClk0 = '1') then
	if (IntProceed_A = '0' and SysTxEna = '1') then
		IntTxRdAdd(4 downto 0) <= IntTxRdAdd(4 downto 0) + "01"; -- 3-bit Binary counter 
		if (IntTxRdAdd(4 downto 0) = "11111") then
			case IntTxRdAdd(8 downto 5) is
				when "0000" => IntTxRdAdd(8 downto 5) <= "0001";
				when "0001" => IntTxRdAdd(8 downto 5) <= "0011";
				when "0011" => IntTxRdAdd(8 downto 5) <= "0010";
				when "0010" => IntTxRdAdd(8 downto 5) <= "0110";
				when "0110" => IntTxRdAdd(8 downto 5) <= "0111";
				when "0111" => IntTxRdAdd(8 downto 5) <= "0101";
				when "0101" => IntTxRdAdd(8 downto 5) <= "0100";
				when "0100" => IntTxRdAdd(8 downto 5) <= "1100";
				when "1100" => IntTxRdAdd(8 downto 5) <= "1101";
				when "1101" => IntTxRdAdd(8 downto 5) <= "1111";
				when "1111" => IntTxRdAdd(8 downto 5) <= "1110";
				when "1110" => IntTxRdAdd(8 downto 5) <= "1010";
				when "1010" => IntTxRdAdd(8 downto 5) <= "1011";
				when "1011" => IntTxRdAdd(8 downto 5) <= "1001";
				when "1001" => IntTxRdAdd(8 downto 5) <= "1000";
				when others => IntTxRdAdd(8 downto 5) <= "0000";
			end case;
		end if;
		if (IntTxRdAdd(8 downto 5) = "1000" and IntTxRdAdd(4 downto 0) = "11111") then
			case IntTxRdAdd(11 downto 9) is
				when "000" => IntTxRdAdd(11 downto 9) <= "001";
				when "001" => IntTxRdAdd(11 downto 9) <= "011";
				when "011" => IntTxRdAdd(11 downto 9) <= "010";
				when "010" => IntTxRdAdd(11 downto 9) <= "110";
				when "110" => IntTxRdAdd(11 downto 9) <= "111";
				when "111" => IntTxRdAdd(11 downto 9) <= "101";
				when "101" => IntTxRdAdd(11 downto 9) <= "100";
				when others => IntTxRdAdd(11 downto 9) <= "000";
			end case;
		end if;
	end if;
end if;
end process;
--
GenOne : for n in 9 to 11 generate
	MsbTxRdAddrFf : FDC
		port map (
			D	=> IntTxRdAdd(n),
			C	=> SysClk,
			CLR	=> SysRst,
			Q	=> IntTxRdAddFf(n)
			);
end generate;
--
-- 8-bit wide flag system.
-- Used for fine tuning of the system.
--
TxFifoFlags : process (SysClk, SysRst, IntTxRdAddFf(11 downto 9), IntTxWrAdd(8 downto 6))
subtype TxRdWrAddr is std_logic_vector (5 downto 0);
begin
	if (SysRst = '1') then
		SysTxFifoStat <= (others => '0');
	elsif (SysClk'event and SysCLk = '1') then
		case TxRdWrAddr'(IntTxRdAddFf(11 downto 9) & IntTxWrAdd(8 downto 6)) is
			--  Read Write								-- Between 7/8 and full
			when "000100" => SysTxFifoStat(7) <= '1';
			when "001000" => SysTxFifoStat(7) <= '1';
			when "011001" => SysTxFifoStat(7) <= '1';
			when "010011" => SysTxFifoStat(7) <= '1';
			when "110010" => SysTxFifoStat(7) <= '1';
			when "111110" => SysTxFifoStat(7) <= '1';
			when "101111" => SysTxFifoStat(7) <= '1';
			when "100101" => SysTxFifoStat(7) <= '1';
			when others => SysTxFifoStat(7) <= '0';
		end case;
		case TxRdWrAddr'(IntTxRdAddFf(11 downto 9) & IntTxWrAdd(8 downto 6)) is
			--  Read Write								-- Between 6/8 and 7/8
			when "000101" => SysTxFifoStat(6) <= '1';
			when "001100" => SysTxFifoStat(6) <= '1';
			when "011000" => SysTxFifoStat(6) <= '1';
			when "010001" => SysTxFifoStat(6) <= '1';
			when "110011" => SysTxFifoStat(6) <= '1';
			when "111010" => SysTxFifoStat(6) <= '1';
			when "101110" => SysTxFifoStat(6) <= '1';
			when "100111" => SysTxFifoStat(6) <= '1';
			when others => SysTxFifoStat(6) <= '0';
		end case;
		case TxRdWrAddr'(IntTxRdAddFf(11 downto 9) & IntTxWrAdd(8 downto 6)) is
			--  Read Write								-- Between 5/6 and 6/8
			when "000111" => SysTxFifoStat(5) <= '1';
			when "001101" => SysTxFifoStat(5) <= '1';
			when "011100" => SysTxFifoStat(5) <= '1';
			when "010000" => SysTxFifoStat(5) <= '1';
			when "110001" => SysTxFifoStat(5) <= '1';
			when "111011" => SysTxFifoStat(5) <= '1';
			when "101010" => SysTxFifoStat(5) <= '1';
			when "100110" => SysTxFifoStat(5) <= '1';
			when others => SysTxFifoStat(5) <= '0';
		end case;
		case TxRdWrAddr'(IntTxRdAddFf(11 downto 9) & IntTxWrAdd(8 downto 6)) is
			--  Read Write								-- Between 4/8 and 5/8
			when "000110" => SysTxFifoStat(4) <= '1';
			when "001111" => SysTxFifoStat(4) <= '1';
			when "011101" => SysTxFifoStat(4) <= '1';
			when "010100" => SysTxFifoStat(4) <= '1';
			when "110000" => SysTxFifoStat(4) <= '1';
			when "111001" => SysTxFifoStat(4) <= '1';
			when "101011" => SysTxFifoStat(4) <= '1';
			when "100010" => SysTxFifoStat(4) <= '1';
			when others => SysTxFifoStat(4) <= '0';
		end case;
		case TxRdWrAddr'(IntTxRdAddFf(11 downto 9) & IntTxWrAdd(8 downto 6)) is
			--  Read Write								-- Between 3/8 and 4/8
			when "000010" => SysTxFifoStat(3) <= '1';
			when "001110" => SysTxFifoStat(3) <= '1';
			when "011111" => SysTxFifoStat(3) <= '1';
			when "010101" => SysTxFifoStat(3) <= '1';
			when "110100" => SysTxFifoStat(3) <= '1';
			when "111000" => SysTxFifoStat(3) <= '1';
			when "101001" => SysTxFifoStat(3) <= '1';
			when "100011" => SysTxFifoStat(3) <= '1';
			when others => SysTxFifoStat(3) <= '0';
		end case;
		case TxRdWrAddr'(IntTxRdAddFf(11 downto 9) & IntTxWrAdd(8 downto 6)) is
			--  Read Write								-- Between 2/8 and 3/8
			when "000011" => SysTxFifoStat(2) <= '1';
			when "001010" => SysTxFifoStat(2) <= '1';
			when "011110" => SysTxFifoStat(2) <= '1';
			when "010111" => SysTxFifoStat(2) <= '1';
			when "110101" => SysTxFifoStat(2) <= '1';
			when "111100" => SysTxFifoStat(2) <= '1';
			when "101000" => SysTxFifoStat(2) <= '1';
			when "100001" => SysTxFifoStat(2) <= '1';
			when others => SysTxFifoStat(2) <= '0';
		end case;
		case TxRdWrAddr'(IntTxRdAddFf(11 downto 9) & IntTxWrAdd(8 downto 6)) is
			--  Read Write								-- Between 1/8 and 2/8
			when "000001" => SysTxFifoStat(1) <= '1';
			when "001011" => SysTxFifoStat(1) <= '1';
			when "011010" => SysTxFifoStat(1) <= '1';
			when "010110" => SysTxFifoStat(1) <= '1';
			when "110111" => SysTxFifoStat(1) <= '1';
			when "111101" => SysTxFifoStat(1) <= '1';
			when "101100" => SysTxFifoStat(1) <= '1';
			when "100000" => SysTxFifoStat(1) <= '1';
			when others => SysTxFifoStat(1) <= '0';
		end case;
		case TxRdWrAddr'(IntTxRdAddFf(11 downto 9) & IntTxWrAdd(8 downto 6)) is
			--  Read Write								-- Between 1/8 and empty
			when "000000" => SysTxFifoStat(0) <= '1';
			when "001001" => SysTxFifoStat(0) <= '1';
			when "011011" => SysTxFifoStat(0) <= '1';
			when "010010" => SysTxFifoStat(0) <= '1';
			when "110110" => SysTxFifoStat(0) <= '1';
			when "111111" => SysTxFifoStat(0) <= '1';
			when "101101" => SysTxFifoStat(0) <= '1';
			when "100100" => SysTxFifoStat(0) <= '1';
			when others => SysTxFifoStat(0) <= '0';
		end case;
	end if;
end process;
---------------------------------------------------------------------------------------
-- OSERDES set of components.
-- A separate OSERDES has be instantiated, together with its OBUF for the LxCLKOUT signals.
-- The outputs are differential signals.
---------------------------------------------------------------------------------------
GenTwo :  for n in 3 downto 0 generate
	LxDatoSerdes : OSERDES
		generic map (
			DATA_RATE_OQ	=> "DDR",
			DATA_RATE_TQ	=> "BUF",
			DATA_WIDTH		=> 4,
			INIT_OQ			=> '0',
			INIT_TQ			=> '0',
			SERDES_MODE		=> "MASTER",
			SRVAL_OQ		=> '0',
			SRVAL_TQ		=> '0',
			TRISTATE_WIDTH	=> 1
		)
		port map (
			OQ			=> IntLxDatO(n),
			SHIFTOUT1	=> open,
			SHIFTOUT2	=> open,
			TQ			=> open,
			CLK			=> IntClk0,
			CLKDIV		=> IntProceedNot_B,
			D1 			=> IntTxDat(n),		-- Data bit 0
			D2 			=> IntTxDat(n+4),
			D3 			=> IntTxDat(n+8),
			D4 			=> IntTxDat(n+12),
			D5 			=> Low,
			D6 			=> Low,
			OCE 		=> High,
			REV 		=> Low,
			SHIFTIN1 	=> Low,
			SHIFTIN2 	=> Low,
			SR 			=> SysTxEna,
			T1 			=> Low,
			T2 			=> Low,
			T3 			=> Low,
			T4 			=> Low,
			TCE 		=> Low
		);
	DatOut : OBUFDS_LVDS_25
		port map (
			O	=> LxDATO_p(n),
			OB	=> LxDATO_n(n),
			I	=> IntLxDato(n)
		);
end generate;
---------------------------------------------------------------------------------------
-- OSERDES for LxCLKOUT
---------------------------------------------------------------------------------------
LxClkOserdes : OSERDES
	generic map (
		DATA_RATE_OQ	=> "DDR",
		DATA_RATE_TQ	=> "BUF",
		DATA_WIDTH		=> 4,
		INIT_OQ			=> '0',
		INIT_TQ			=> '0',
		SERDES_MODE		=> "MASTER",
		SRVAL_OQ		=> '0',
		SRVAL_TQ		=> '0',
		TRISTATE_WIDTH	=> 1
	)
	port map (
		OQ			=> IntLxClkOut,
		SHIFTOUT1	=> open,
		SHIFTOUT2	=> open,
		TQ			=> open,
		CLK			=> IntClk90,
		CLKDIV		=> IntProceed90,
		D1 			=> High,			-- LxOUTCLK Clock pattern bit 0
		D2 			=> Low,
		D3 			=> High,
		D4 			=> Low,
		D5 			=> Low,
		D6 			=> Low,
		OCE 		=> High,
		REV 		=> Low,
		SHIFTIN1 	=> Low,
		SHIFTIN2 	=> Low,
		SR 			=> SysTxEna,
		T1 			=> Low,
		T2 			=> Low,
		T3 			=> Low,
		T4 			=> Low,
		TCE 		=> Low
	);
ClkOut : OBUFDS_LVDS_25
	port map (
		O	=> LxCLKOUTp,
		OB	=> LxCLKOUTn,
		I	=> IntLxclkOut
	);
---------------------------------------------------------------------------------------
--
end  LinkPortTx_arch ;