------------------------------------------------------------------------------------------------------------------------------
-- Entity Name:  Toplevel
-- File Name:    Toplevel.vhd
-- File Path:    \Projects\TigerSharc\Vhdl\FeaturedDesign\Toplevel
-- Project :	TigerSharc Link Port Interface for Virtex-4
--
-- Purpose: 
--
-- Limitations: none
--
-- Authors: Marc Defossez
--
-- Tools: 
--
--  Contact:    e-mail  hotline@xilinx.com        phone   + 1 800 255 7778
--
--  Disclaimer:
--              LIMITED WARRANTY AND DISCLAIMER.
--              These designs are provided to you "as is". Xilinx and its licensors make and you
--              receive no warranties or conditions, express, implied, statutory or otherwise, and
--              Xilinx specifically disclaims any implied warranties of merchantability, non-infringement,
--              or fitness for a particular purpose. Xilinx does not warrant that the functions contained
--              in these designs will meet your requirements, or that the operation of these designs will
--              be uninterrupted or error free, or that defects in the designs will be corrected.
--              Furthermore, Xilinx does not warrant or make any representations regarding use or the
--              results of the use of the designs in terms of correctness, accuracy, reliability, or otherwise.
--
--              LIMITATION OF LIABILITY.
--              In no event will Xilinx or its licensors be liable for any loss of data, lost profits, cost or
--              procurement of substitute goods or services, or for any special, incidental, consequential,
--              or indirect damages arising from the use or operation of the designs or accompanying
--              documentation, however caused and on any theory of liability. This limitation will apply
--              even if Xilinx has been advised of the possibility of such damage. This limitation shall
--              apply not-withstanding the failure of the essential purpose of any limited remedies herein.
--
--              Copyright  2002 Xilinx, Inc.
--              All rights reserved
--
-- Revision History:       Created:       27/04/05
--                         Last opened:   Wednesday,Apr 27 05
--
-------------------------------------------------------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_UNSIGNED.all;
-- Simulation libraries
	-- synopsys translate_off
	library UNISIM;
	use UNISIM.all;
	-- synopsys translate_on
-- end simulation libraries

-- Libraries for Synplicity
-- Needed for components as LUT, SRL which get an INIT parameter.
-- When this library is not mentioned Synplify_Pro fails during synthesis of
-- the hardware with an error about the INIT value.
-- library virtex2;
-- use virtex2.components.all;
-- This library is ignored by Mentor Precision.
-- Only a warning is generated.
-- End libraries for Synplicity

entity Toplevel is
    port (
    	-- System ports
    	SystemClkP		: in std_logic;
    	SystemClkN		: in std_logic;
    	SystemRefClk	: in std_logic;
    	TxClkP			: in std_logic;
    	TxClkN			: in std_logic;
    	SystemRst		: in std_logic;
    	SystemTxBcmpo	: in std_logic;
    	SystemRxAcko	: in std_logic;
    	SystemRxEna		: in std_logic;
    	SystemRxFifoOe	: in std_logic;
    	SystemRxBcmpi	: out std_logic;
    	SystemRxFifoStat : out std_logic_vector (3 downto 0);
    	SystemTxFifoStat : out std_logic_vector (7 downto 0);
    	SystemRxData	 : out std_logic_vector (127 downto 0);
		-- Link Port Receiver
		LxCLKINp		: in std_logic;
		LxCLKINn		: in std_logic;
		LxDATI_p		: in std_logic_vector (3 downto 0);
		LxDATI_n		: in std_logic_vector (3 downto 0);
		LxACKO			: out std_logic;
		LxBCMPIn		: in std_logic;
        -- Link Port Transmitter
        LxCLKOUTp	: out std_logic;
        LxCLKOUTn	: out std_logic;
        LxDATO_p	: out std_logic_vector (3 downto 0);
        LxDATO_n	: out std_logic_vector (3 downto 0);
        LxACKI		: in std_logic;
        LxBCMPOn	: out std_logic
		
    );
end  Toplevel;

architecture Toplevel_arch of Toplevel  is
-------------------------------------------------------------------------------------------------------------------------------
-- Component Instantiation
-------------------------------------------------------------------------------------------------------------------------------
component IBUFGDS	generic (DIFF_TERM : boolean := FALSE;
							 IOSTANDARD : string := "DEFAULT");
					port (I, IB : in std_logic; O  : out std_logic);	end component;
component BUFG		port (I : in std_logic; O : out std_logic);	end component;
component IBUF_LVCMOS25	port (O : out std_logic; I : in  std_logic); end component;
component FDP		generic (INIT : bit := '1');
					port (C, D, PRE : in std_logic; Q : out std_logic); end component;

component DCM_ADV
   generic (
		CLK_FEEDBACK 			: string :=  "1X";
		CLKDV_DIVIDE 			: real :=  2.000000;
		CLKFX_DIVIDE 			: integer :=  1;
		CLKFX_MULTIPLY 			: integer :=  4;
		CLKIN_DIVIDE_BY_2 		: boolean :=  FALSE;
		CLKIN_PERIOD 			: real :=  0.000000;
		CLKOUT_PHASE_SHIFT 		: string :=  "NONE";
		DCM_PERFORMANCE_MODE 	: string :=  "MAX_SPEED";
		DESKEW_ADJUST 			: string :=  "SYSTEM_SYNCHRONOUS";
		DFS_FREQUENCY_MODE 		: string :=  "LOW";
		DLL_FREQUENCY_MODE 		: string :=  "LOW";
		DUTY_CYCLE_CORRECTION 	: boolean :=  TRUE;
		PHASE_SHIFT 			: integer :=  0;
		STARTUP_WAIT 			: boolean :=  FALSE;
		FACTORY_JF 				: bit_vector :=  x"F0F0"
    );
	port (
		CLKIN    : in    std_logic; 
		CLKFB    : in    std_logic; 
		DADDR    : in    std_logic_vector (6 downto 0); 
		DI       : in    std_logic_vector (15 downto 0); 
		DWE      : in    std_logic; 
		DEN      : in    std_logic; 
		DCLK     : in    std_logic; 
		RST      : in    std_logic; 
		PSEN     : in    std_logic; 
		PSINCDEC : in    std_logic; 
		PSCLK    : in    std_logic; 
		CLK0     : out   std_logic; 
		CLK90    : out   std_logic; 
		CLK180   : out   std_logic; 
		CLK270   : out   std_logic; 
		CLKDV    : out   std_logic; 
		CLK2X    : out   std_logic; 
		CLK2X180 : out   std_logic; 
		CLKFX    : out   std_logic; 
		CLKFX180 : out   std_logic; 
		DRDY     : out   std_logic; 
		DO       : out   std_logic_vector (15 downto 0); 
		LOCKED   : out   std_logic; 
		PSDONE   : out   std_logic
    );
end component;

component LinkPortRx
    port (
    	-- General
    	SysClk			: in std_logic;
    	SysRefClk		: in std_logic;
    	SystemRst		: in std_logic;		-- System reset used for rest of IDELAY_CTRL
    	SysRst			: in std_logic;		-- Reset of the logic after lock of DCM's and IDELAY_CTRL
    	SysLxAcko		: in std_logic;
    	SysRxFifoOe		: in std_logic;
    	SysRxEna		: in std_logic;
    	SysLxBcmpiN		: out std_logic;
    	SysRxIdlyCtrlRdy : out std_logic;
    	SysRxData		: out std_logic_vector (127 downto 0);
    	SysRxFifoStat 	: out std_logic_vector (3 downto 0);
		-- Link Port Receiver
		LxCLKINp		: in std_logic;
		LxCLKINn		: in std_logic;
		LxDATI_p		: in std_logic_vector (3 downto 0);
		LxDATI_n		: in std_logic_vector (3 downto 0);
		LxACKO			: out std_logic;
		LxBCMPIn		: in std_logic
		);
end component;

component LinkPortTx
    port (
    	-- general
        SysTxClkP	: in std_logic;		-- Clock to DCM for transmitter.
        SysTxClkN	: in std_logic;
        SysClk		: in std_logic;		-- System clock from a system DCM
        SystemRst	: in std_logic;		-- System Reset
        SysRst		: in std_logic;		-- Reset after DCM's have been locked.
        SysTxDcmLckd : out std_logic;	-- locked condition of the transmitter DCM.
        SysTxFifoEna : in std_logic;		-- BRAM Memory enable from application side.
        SysTxEna	: in std_logic;		-- Transmitter enable.
        SysLxBcmpoN	: in std_logic;		-- LxBCMPIn passed from back-end design to transmitter.
        SysLxAcki	: out std_logic;	-- LxACKI signal passed through to the application design.
        SysTxData	: in std_logic_vector (127 downto 0);
        SysTxFifoStat : out std_logic_vector (7 downto 0); -- Fine grained status FIFO status bits.
        -- Link Port Transmitter
        LxCLKOUTp	: out std_logic;
        LxCLKOUTn	: out std_logic;
        LxDATO_p	: out std_logic_vector (3 downto 0);
        LxDATO_n	: out std_logic_vector (3 downto 0);
        LxACKI		: in std_logic;
        LxBCMPOn	: out std_logic
    );
end component;
-------------------------------------------------------------------------------------------------------------------------------
-- Signal, Constants and Attributes Declarations
------------------------------------------------------------------------------------------------------------------------------
-- Constants

-- Signals
signal Low              : std_logic;
signal LowSixTeen		: std_logic_vector (15 downto 0);
signal LowSeven			: std_logic_vector (6 downto 0);
signal High				: std_logic;

signal IntIbufgToSysDcm	: std_logic;
signal IntSysClk0ToBufg	: std_logic;
signal IntSysClk0		: std_logic;
signal IntSystemRst		: std_logic;
signal IntSysRst		: std_logic;
signal IntSysDcmLocked  : std_logic;
signal IntSysDcmLockedNot	: std_logic;
signal IntTxDcmLocked		: std_logic;
signal IntTxDcmLockedNot	: std_logic;
signal IntRxIdlyCtrlRdy		: std_logic;
signal IntRxIdlyCtrlRdyNot	: std_logic;
signal IntAndSigSysRst		: std_logic;

signal IntTxFifoStat		: std_logic_vector (7 downto 0); -- 7 = Full .... 0 = Empty
signal IntDataVal		: std_logic_vector (127 downto 0);
signal IntWrite			: std_logic;
signal IntPatCnt 		: std_logic_vector (3 downto 0);
signal IntTxLxAcki		: std_logic;
signal IntSrlShft		: std_logic_vector (15 downto 0);
signal IntTxStart		: std_logic;
signal IntTxBcmpoN		: std_logic;
signal IntRxFifoOe		: std_logic;
signal IntRxEna			: std_logic;
signal IntRxData		: std_logic_vector (127 downto 0);
signal IntRxFifoStat	: std_logic_vector (3 downto 0);
signal IntRxAcko		: std_logic;
signal IntRxBcmpiN		: std_logic;

-- Attributes

------------------------------------------------------------------------------------------------------------------------------
--
begin
---------------------------------------------------------------------------------------
-- Stuff
---------------------------------------------------------------------------------------
Low <= '0';
LowSixTeen <= X"0000";
High <= '1';
---------------------------------------------------------------------------------------
-- Inputs and Outputs.
-- Non dedicated inputs and outputs are mentionned here.
-- All FPGA inputs and outputs of the transmitter and receiver have been instantiated
-- in the respective components.
---------------------------------------------------------------------------------------
TheSystemRstIbuf : IBUF_LVCMOS25
	port map (
		O => IntSystemRst,
		I => SystemRst		-- Reset the DCM's and the SysRstFf (reset of the logic)
	);
---------------------------------------------------------------------------------------
-- System clocking
---------------------------------------------------------------------------------------
TheSystemIbufgds : IBUFGDS
	generic map (
		DIFF_TERM	=>	FALSE,
		IOSTANDARD	=>	"LVDS_25"
	)
	port map (
		I	=> SystemClkP,
		IB	=> SystemClkN,
		O	=> IntIbufgToSysDcm
	);
             
TheSystemDcm : DCM_ADV
	generic map (
		CLK_FEEDBACK 		=> "1X",
		CLKDV_DIVIDE 		=> 4.000000,
		CLKFX_DIVIDE 		=> 1,
		CLKFX_MULTIPLY 		=> 4,
		CLKIN_DIVIDE_BY_2 	=> FALSE,
		CLKIN_PERIOD 		=> 2.222220,
		CLKOUT_PHASE_SHIFT 	=> "NONE",
		DCM_PERFORMANCE_MODE =>	"MAX_SPEED",
		DESKEW_ADJUST 		=> "SYSTEM_SYNCHRONOUS",
		DFS_FREQUENCY_MODE 	=> "HIGH",
		DLL_FREQUENCY_MODE	=> "HIGH",
		DUTY_CYCLE_CORRECTION => TRUE,
		PHASE_SHIFT 		=> 0,
		STARTUP_WAIT 		=>	FALSE,
		FACTORY_JF 			=> X"F0F0"
	)
	port map (
   		CLKFB		=> IntSysClk0,
		CLKIN		=> IntIbufgToSysDcm,
		DADDR		=> LowSeven,
		DCLK		=> Low,
		DEN			=> Low,
		DI			=> LowSixTeen,
		DWE			=> Low,
		PSCLK		=> Low,
		PSEN		=> Low,
		PSINCDEC	=> Low,
		RST			=> IntSystemRst,
		CLKDV		=> open,
		CLKFX		=> open,
		CLKFX180	=> open,
		CLK0		=> IntSysClk0ToBufg,
		CLK2X		=> open,
		CLK2X180	=> open,
		CLK90		=> open,
		CLK180		=> open,
		CLK270		=> open,
		DO			=> open,
		DRDY		=> open,
		LOCKED		=> IntSysDcmLocked,
		PSDONE		=> open
	);
TheSysClk0Bufg : BUFG	port map (I => IntSysClk0ToBufg, O => IntSysClk0);	-- System Clock

IntSysDcmLockedNot <= not IntSysDcmLocked;
IntTxDcmLockedNot <= not IntTxDcmLocked;
IntRxIdlyCtrlRdyNot <= not IntRxIdlyCtrlRdy;

IntAndSigSysRst <= IntSysDcmLockedNot and IntTxDcmLockedNot and IntRxIdlyCtrlRdyNot;
TheSysRstFf : FDP
	generic map (INIT => '1')
	port map (
		C	=> IntIbufgToSysDcm,
		D   => IntAndSigSysRst,
		PRE => IntSystemRst,
		Q   => IntSysRst		-- This reset acts as reset for the whole logic.
		);
---------------------------------------------------------------------------------------
-- Receiver
---------------------------------------------------------------------------------------
TheRx : LinkPortRx
    port map (
    	-- General
    	SysClk		=> IntSysClk0,
    	SysRefClk	=> SystemRefClk,
    	SystemRst	=> IntSystemRst,	-- System reset used for rest of IDELAY_CTRL
    	SysRst		=> IntSysRst,	-- Reset of the logic after lock of DCM's and IDELAY_CTRL
    	SysLxAcko	=> IntRxAcko,
    	SysRxFifoOe	=> IntRxFifoOe,
    	SysRxEna	=> IntRxEna,
    	SysLxBcmpiN	=> IntRxBcmpiN,
    	SysRxIdlyCtrlRdy => IntRxIdlyCtrlRdy,
    	SysRxData	=> IntRxData,
    	SysRxFifoStat => IntRxFifoStat,
		-- Link Port Receiver
		LxCLKINp	=> LxCLKINp,
		LxCLKINn	=> LxCLKINn,
		LxDATI_p	=> LxDATI_p,
		LxDATI_n	=> LxDATI_n,
		LxACKO		=> LxACKO,
		LxBCMPIn	=> LxBCMPIn 
		);
---------------------------------------------------------------------------------------
-- Transmitter
---------------------------------------------------------------------------------------
TheTx : LinkPortTx
    port map (
    	-- general
        SysTxClkP	=> TxClkP,	-- Clock to DCM for transmitter.
        SysTxClkN	=> TxClkN,
        SysClk		=> IntSysClk0,	-- System clock from a system DCM
        SystemRst	=> IntSystemRst,	-- System Reset
        SysRst		=> IntSysRst,	-- Reset after DCM's have been locked.
        SysTxDcmLckd => IntTxDcmLocked,	-- locked condition of the transmitter DCM.
        SysTxFifoEna => IntWrite ,	-- BRAM Memory enable from application side.
        SysTxEna	=> IntTxStart,	-- Transmitter enable.
        SysLxBcmpoN	=> IntTxBcmpoN,	-- LxBCMPIn passed from back-end design to transmitter.
        SysLxAcki	=> IntTxLxAcki,	-- LxACKI signal passed through to the application design.
        SysTxData	=> IntDataVal,
        SysTxFifoStat => IntTxFifoStat, -- Fine grained status FIFO status bits.
        -- Link Port Transmitter
        LxCLKOUTp	=> LxCLKOUTp,
        LxCLKOUTn	=> LxCLKOUTn,
        LxDATO_p	=> LxDATO_p,
        LxDATO_n	=> LxDATO_n,
        LxACKI		=> LxACKI,
        LxBCMPOn	=> LxBCMPOn	    
    );
---------------------------------------------------------------------------------------
-- Back end design
--
-- A full design has not been foreseen since the back-end design is depending the
-- application used. The small design here is just to prove the HDL code.
---------------------------------------------------------------------------------------
--------------------------------------------------------------------
-- Pattern generator and BRAM fill
--------------------------------------------------------------------
BramWrData : process (IntSysClk0, IntSysRst)
begin
if (IntSysRst = '1') then
	IntDataVal <= (others => '0');
	IntWrite <= '0';
	IntPatCnt <= "0000";
elsif (IntSysClk0'event and IntSysClk0 = '1') then
	if (IntTxFifoStat(7) = '0' and IntTxFifoStat(0) = '1') then
		IntPatCnt <= IntPatCnt + 1;
	elsif (IntTxFifoStat(7) /= '1') then
		IntPatCnt <= IntPatCnt + 1;
		if (IntPatCnt = "1111") then
			IntDataVal <= ((IntDataVal(127 downto 112) - '1') & (IntDataVal(111 downto 96) + '1') &
			         (IntDataVal(95 downto 80) - '1') & (IntDataVal(79 downto 64) + '1') &
			         (IntDataVal(63 downto 48) - '1') & (IntDataVal(47 downto 32) + '1') &
			         (IntDataVal(31 downto 16) - '1') & (IntDataVal(15 downto 0) + '1'));
			IntWrite <= '1';
		else
			IntWrite <= '0';
		end if ;
	else 
		IntWrite <= '0';
	end if;
end if ;
end process;
--------------------------------------------------------------------
-- Transmission Start
--------------------------------------------------------------------
-- Sample LxAcki and wait for x clock cycles to start the transmitter.
SampleLxAcki : process (IntSysClk0, IntTxLxAcki)
begin
if (IntSysClk0'event and IntSysClk0 = '1') then
	IntSrlShft <= IntTxLxAcki & IntSrlShft(15 downto 1);
end if;

end process;
-- Start the transmitter when LxAcki is high and the transmit fifo is full or a start command is given.
TxStart : process (IntSysClk0, IntSysRst)
begin
if (IntSysRst = '1') then
	IntTxStart <= '0';
elsif (IntSysClk0'event and IntSysClk0 = '1') then
	if (IntSrlShft(0) = '1' and IntTxFifoStat(7) = '1') then
		IntTxStart <= '1';
	end if;
end if;
end process;

SystemFfs : process (IntSysClk0, IntSysRst)
begin
if (IntSysRst = '1') then
	IntRxAcko <= '0';
	IntTxBcmpoN <= '0';
	SystemRxBcmpi <= '0';
	IntRxEna <= '0';
	IntRxFifoOe <= '0';
	SystemRxFifoStat <= "0000";
	SystemTxFifoStat <= "00000000";
	SystemRxData <= (others => '0');
elsif (IntSysClk0'event and IntSysClk0 = '1') then
	IntRxAcko <= SystemRxAcko;
	SystemRxBcmpi <= IntRxBcmpiN;
	IntTxBcmpoN <= SystemTxBcmpo;
	SystemRxData <= IntRxData;
	SystemRxFifoStat <= IntRxFifoStat;
	SystemTxfifoStat <= IntTxFifoStat;
	IntRxEna <= SystemRxEna;
	IntRxFifoOe <= SystemRxFifoOe;
end if;
end process;

--
end  Toplevel_arch ;