-----------------------------------------------------------------------------------------------------------------------------------------
-- Entity Name:  RxBytAndQwrdCnt
-- File Name:    RxBytAndQwrdCnt.vhd
-- File Path:    D:\Projects\TigerSharc\Vhdl\Virtex4\
-- Project :
--
-- Purpose: 	Gray counter counting shifted bytes and Quad words.
--				Control registers keeping track of the past.
--
-- Limitations: none
--
-- Authors: Marc Defossez
--
-- Tools: 
--
--  Contact:    e-mail  hotline@xilinx.com        phone   + 1 800 255 7778
--
--  Disclaimer:
--              LIMITED WARRANTY AND DISCLAIMER.
--              These designs are provided to you "as is". Xilinx and its licensors make and you
--              receive no warranties or conditions, express, implied, statutory or otherwise, and
--              Xilinx specifically disclaims any implied warranties of merchantability, non-infringement,
--              or fitness for a particular purpose. Xilinx does not warrant that the functions contained
--              in these designs will meet your requirements, or that the operation of these designs will
--              be uninterrupted or error free, or that defects in the designs will be corrected.
--              Furthermore, Xilinx does not warrant or make any representations regarding use or the
--              results of the use of the designs in terms of correctness, accuracy, reliability, or otherwise.
--
--              LIMITATION OF LIABILITY.
--              In no event will Xilinx or its licensors be liable for any loss of data, lost profits, cost or
--              procurement of substitute goods or services, or for any special, incidental, consequential,
--              or indirect damages arising from the use or operation of the designs or accompanying
--              documentation, however caused and on any theory of liability. This limitation will apply
--              even if Xilinx has been advised of the possibility of such damage. This limitation shall
--              apply not-withstanding the failure of the essential purpose of any limited remedies herein.
--
--              Copyright  2002 Xilinx, Inc.
--              All rights reserved
--
-- Revision History:       Created:       25/04/05
--                         Last opened:   Monday,Apr 25 05
--
----------------------------------------------------------------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_UNSIGNED.all;
-- Simulation libraries
	library UNISIM;
	use UNISIM.all;
-- end simulation libraries

-- Libraries for Synplicity
-- Needed for components as LUT, SRL which get an INIT parameter.
-- When this library is not mentioned Synplify_Pro fails during synthesis of
-- the hardware with an error about the INIT value.
-- library virtex2;
-- use virtex2.components.all;
-- This library is ignored by Mentor Precision.
-- Only a warning is generated.
-- End libraries for Synplicity

entity RxBytAndQwrdCnt is
    port (
        Clk : in std_logic;
        EnaMux : in std_logic;
        EnaMuxN : in std_logic;
        Rst : in std_logic;
        ClkDiv	: out std_logic
    );
end  RxBytAndQwrdCnt ;

architecture RxBytAndQwrdCnt_arch of RxBytAndQwrdCnt  is
----------------------------------------------------------------------------------------------------------------------------------------
-- Component Instantiation
----------------------------------------------------------------------------------------------------------------------------------------
component FDCE		port (Q : out std_logic; D, C, CE, CLR : in std_logic); end component;
component FD		port (Q : out std_logic; D, C : in std_logic); end component;
component FD_1		port (Q : out std_logic; D, C : in std_logic); end component;
component LUT2
	generic (INIT : bit_vector (3 downto 0));
	port (I0, I1 : in std_logic; O : out std_logic); end component;
component LUT4
	generic (INIT : bit_vector (15 downto 0));
	port (I0, I1, I2, I3 : in std_logic; O : out std_logic); end component;
----------------------------------------------------------------------------------------------------------------------------------------
-- Signal, Constants and Attributes Declarations
----------------------------------------------------------------------------------------------------------------------------------------
-- Constants
type InitIncVal2b is array (1 downto 0) of bit_vector (3 downto 0);
constant IncLutVal2b : InitIncVal2b := (
	X"A",	-- Address one
	X"3"	-- Address zero
	);
constant Tc2bMuxLutVal	: bit_vector (15 downto 0) := X"8111"; -- IntLxClkInDiv Multiplexer.

type InitIncVal4b is array (3 downto 0) of bit_vector (15 downto 0);
constant IncLutVal4b : InitIncVal4b := (
	X"FE10",	-- Address three
	X"B0F4",
	X"E44E",
	X"3CC3"		-- Address zero
	);
constant Tc4bLutVal : bit_vector (15 downto 0) := X"0200";  -- Terminal Count of Gray counter.

-- Signals
signal High 			: std_logic;
signal Low	 			: std_logic;
signal IntByteCntOut	: std_logic_vector (1 downto 0);
signal IntByteIncOut 	: std_logic_vector (1 downto 0);
signal IntClkDiv		: std_logic;

signal IntQwrdCntOut	: std_logic_vector (3 downto 0);
signal IntQwrdIncOut 	: std_logic_vector (3 downto 0);
signal IntQWrdTc		: std_logic;
signal IntQWrdTcDlyOne	: std_logic;
signal IntQWrdTcDlyTwo	: std_logic;

signal IntQwrdTcPropFfEna : std_Logic;
signal IntQwrdMuxEna : std_logic;

-- attributes
attribute RLOC : string;
	type RlocIncVal2b is array (1 downto 0) of string (4 downto 1);
	constant IncRlocVal2b : RlocIncVal2b := (
		"X0Y0",		-- Bit 1 (LUT and FF)
		"X0Y0"		-- Bit 0
	);
	type RlocValue4b is array (3 downto 0) of string (4 downto 1);
	constant RlocVal4b : RlocValue4b := (
		"X2Y1",		-- Bit 3
		"X2Y1",		-- Bit 2
		"X2Y0",		-- Bit 1
		"X2Y0"		-- Bit 0
	);
	attribute RLOC of ByteCntTcAndMuxLut 	: label is "X0Y1";
	attribute RLOC of TcQwrdLut 			: label is "X0Y1";
	attribute RLOC of TheQwrdTcDlyOneFf		: label is "X1Y0";
--	attribute RLOC of TheQwrdTcDlyTwoFf		: label is "X1Y1";

----------------------------------------------------------------------------------------------------------------------------------------
--
Begin
Low <= '0';
High <= '1';
-------------------------------------------------------------------
-- Byte counter, Gray counter.
-------------------------------------------------------------------
GenByteCnt : for i in 1 downto 0 generate
	attribute RLOC of ByteIncrLut : label is IncRlocVal2b(i);
	attribute RLOC of ByteIncrFf : label is IncRlocVal2b(i);
	begin
	ByteIncrLut : LUT2
	generic map (
		INIT => IncLutVal2b(i)
	)
	port map (
		I0 	=> IntByteCntOut(0),
		I1 	=> IntByteCntOut(1),
		O 	=> IntByteIncOut(i)
	);
	ByteIncrFf : FD
	port map (
		D	=> IntByteIncOut(i),
		C	=> Clk,
		Q	=> IntByteCntOut(i)
	);
end generate;

-------------------------------------------------------------------
-- Byte counter terminal count and slow (CLKDIV) clock multiplexer.
-- Delay for LxACKO and RVERE status bits, enabling the multiplexer.
-------------------------------------------------------------------
IntQwrdTcPropFfEna <= EnaMuxN and EnaMux;

TheQwrdTcDlyOneFf : FD_1		-- Clocked one negative edge
	port map (
		D	=> IntQwrdTc,
		C	=> Clk,
		Q	=> IntQwrdTcDlyOne
	);
--TheQWrdTcDlyTwoFf : FD			-- Clocked on positive edge
--	port map (
--		D	=> IntQwrdTcDlyOne,
--		C	=> Clk,
--		Q	=> IntQwrdTcDlyTwo
--	);

ByteCntTcAndMuxLut : LUT4
	generic map (
		INIT => Tc2bMuxLutVal
	)
	port map (
		I0 	=> IntByteCntOut(0),
		I1 	=> IntByteCntOut(1),
		I2 	=> IntQwrdTcDlyOne,
		I3 	=> IntQwrdTcPropFfEna,
		O 	=> ClkDiv
	);

-------------------------------------------------------------------
-- Quad word counter enabled every four bytes.
-------------------------------------------------------------------
GenQwrdCnt : for i in 3 downto 0 generate
	attribute RLOC of QwrdIncrLut : label is RlocVal4b(i);
	attribute RLOC of QwrdIncrFf : label is RlocVal4b(i);
	begin
	QwrdIncrLut : LUT4
	generic map (
		INIT => IncLutVal4b(i)
	)
	port map (
		I0 	=> IntQwrdCntOut(0),
		I1 	=> IntQwrdCntOut(1),
		I2 	=> IntQwrdCntOut(2),
		I3 	=> IntQwrdCntOut(3),
		O 	=> IntQwrdIncOut(i)
	);
	QwrdIncrFf : FD
	port map (
		D	=> IntQwrdIncOut(i),
		C	=> Clk,
		Q	=> IntQwrdCntOut(i)
	);
end generate;

TcQwrdLut : LUT4
	generic map (
		INIT => Tc4bLutVal
	)
	port map (
		I0	=> IntQwrdCntOut(0),
		I1	=> IntQwrdCntOut(1),
		I2	=> IntQwrdCntOut(2),
		I3	=> IntQwrdCntOut(3),
		O	=> IntQwrdTc
	);
-------------------------------------------------------------------
end  RxBytAndQwrdCnt_arch ;