------------------------------------------------------------------------------------------------------------------------
-- Copyright (c) 1996-2003 Xilinx, Inc.
-- All Rights Reserved
------------------------------------------------------------------------------------------------------------------------
--
--  Disclaimer:
--              LIMITED WARRANTY AND DISCLAIMER.
--              These designs are provided to you "as is". Xilinx and its licensors make and you
--              receive no warranties or conditions, express, implied, statutory or otherwise, and
--              Xilinx specifically disclaims any implied warranties of merchantability, non-infringement,
--              or fitness for a particular purpose. Xilinx does not warrant that the functions contained
--              in these designs will meet your requirements, or that the operation of these designs will
--              be uninterrupted or error free, or that defects in the designs will be corrected.
--              Furthermore, Xilinx does not warrant or make any representations regarding use or the
--              results of the use of the designs in terms of correctness, accuracy, reliability, or otherwise.
--
--              LIMITATION OF LIABILITY.
--              In no event will Xilinx or its licensors be liable for any loss of data, lost profits, cost or
--              procurement of substitute goods or services, or for any special, incidental, consequential,
--              or indirect damages arising from the use or operation of the designs or accompanying
--              documentation, however caused and on any theory of liability. This limitation will apply
--              even if Xilinx has been advised of the possibility of such damage. This limitation shall
--              apply not-withstanding the failure of the essential purpose of any limited remedies herein.
--
--  Contact:    e-mail  hotline@xilinx.com        phone   + 1 800 255 7778
--   ____  ____
--  /   /\/   /
-- /___/  \  /   Vendor: Xilinx
-- \   \   \/    Version: 1.0.0
--  \   \        Filename: TxBytAndQwrdCnt.vhd
--  /   /        Date Last Modified:  21/06/05
-- /___/   /\    Date Created: 16/06/05
-- \   \  /  \
--  \___\/\___\
-- 
-- Device: XC4VLX40-11FF1148
-- Author: Marc Defossez
-- Entity Name:  TxBytAndQwrdCnt
-- Purpose: Create a Virtex-4 to/from ADSP-TS201x LinkPort.
-- Tools: ISE7.1.02
-- Limitations: none
--
-- Revision History:
--    Rev. 
--
------------------------------------------------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_UNSIGNED.all;
-- Simulation libraries
	-- synopsys translate_off
	library UNISIM;
	use UNISIM.all;
	-- synopsys translate_on
-- end simulation libraries

-- Libraries for Synplicity
-- Needed for components as LUT, SRL which get an INIT parameter.
-- When this library is not mentioned Synplify_Pro fails during synthesis of
-- the hardware with an error about the INIT value.
-- library virtex2;
-- use virtex2.components.all;
-- This library is ignored by Mentor Precision.
-- Only a warning is generated.
-- End libraries for Synplicity

entity TxBytAndQwrdCnt is
    port (
        EnaIn		: in std_logic;		-- Enable input from the back-end system. Delayed LxACKI signal
		Clock0		: in std_logic;		-- Tranmit clock from a DCM.
		EnaOut		: out std_logic;	-- Enable for the OSERDES components.
		AsynReset	: out std_logic;	-- Synchrounous reset signal for the OSERDES (must be delayed by one clock cycle)
--		ByteLoad	: out std_logic;	-- Signal to allow a byte to the OSERDES
		ByteClock	: out std_logic		-- OSERDES.CLKDIV.
    );
end  TxBytAndQwrdCnt ;

architecture TxBytAndQwrdCnt_arch of TxBytAndQwrdCnt  is
------------------------------------------------------------------------------------------------------------------------
-- Component Instantiation
------------------------------------------------------------------------------------------------------------------------
component FDRE		port (Q : out std_logic; D, C, CE, R : in std_logic); end component;
component FDRE_1	port (Q : out std_logic; D, C, CE, R : in std_logic); end component;
component FD		port (Q : out std_logic; D, C : in std_logic); end component;
component LUT4		generic (INIT : bit_vector (15 downto 0));
					port (I0, I1, I2, I3 : in std_logic; O : out std_logic); end component;
component LUT2		generic (INIT : bit_vector (3 downto 0));
					port (I0, I1 : in std_logic; O : out std_logic); end component;
------------------------------------------------------------------------------------------------------------------------
-- Signal, Constants and Attributes Declarations
------------------------------------------------------------------------------------------------------------------------
-- Constants
constant TxEnaAndLoadLutVal	: bit_vector (15 downto 0) := X"1110";
type TxByteInitVal is array (1 downto 0) of bit_vector (15 downto 0);
constant TxByteLutVal : TxByteInitVal := (
	X"A000",	-- Address one
	X"3000"	-- Address zero
	);
constant TxByteCntTcLutVal	: bit_vector (15 downto 0) := X"1000";		-- byte counter terminal count LUT.

type TxQwrdInitVal is array (3 downto 0) of bit_vector (15 downto 0);
constant TxQwrdLutVal : TxQwrdInitVal := (
	X"FE10",	-- Address three
	X"B0F4",
	X"E44E",
	X"3CC3"		-- Address zero
	);
constant TxQwrdCntTcLutVal 	: bit_vector (15 downto 0) := X"8000";	-- Qword terminl count LUT.

constant TxQwrdLxAckiLutVal : bit_vector (3 downto 0) := X"2";

-- Signals
signal IntEnaAndLoad		: std_logic;
signal IntEnaOut			: std_logic;

signal IntTxByteIncOut		: std_logic_vector(1 downto 0);
signal IntTxByteCntOut		: std_logic_vector(1 downto 0);
signal IntByteClock			: std_logic;

signal IntTxQwrdCntEna		: std_logic;
signal IntTxQwrdIncOut		: std_logic_vector(3 downto 0);
signal IntTxQwrdCntOut		: std_logic_vector(3 downto 0);
signal IntTxQwrdCntTc		: std_logic;
signal IntTxLxAcki			: std_logic;
signal IntTxQwrdEnaLut		: std_logic;
signal IntSyncReset			: std_logic;

signal Low			: std_logic;
signal High			: std_logic;

-- Attributes
attribute RLOC : string;
--attribute RLOC of TheDatLoadLut : label is "X0Y0";
--	attribute RLOC of TheEnaInFfOne : label is "X0Y0";
--
--type RlocTxByteVal is array (1 downto 0) of string (4 downto 1);
--constant TxByteRlocVal : RlocTxByteVal := (
--	"X1Y0",		-- Bit 1 (LUT and FF)
--	"X1Y0"		-- Bit 0
--);
--	attribute RLOC of TheByteCntTcLut : Label is "X0Y1";
--	attribute RLOC of TheByteCntTcFf : Label is "X0Y1";
--
--type RlocTxQwrdVal is array (3 downto 0) of string (4 downto 1);
--constant TxQwrdRlocVal : RlocTxQwrdVal := (
--	"X2Y1",		-- Bit 3 (LUT and FF)
--	"X2Y1",		-- Bit 2
--	"X2Y0",		-- Bit 1
--	"X2Y0"		-- Bit 0
--);
--attribute RLOC of TheQwrdCntTcLut : label is "X1Y1";
--attribute RLOC of TheQwrdCntTcFf : label is "X1Y1";
--	attribute RLOC of TheRstDlyFf : label is "X1Y1";
------------------------------------------------------------------------------------------------------------------------
--
begin

Low <= '0';
High <= '1';

TheDatLoadLut : LUT4
generic map (
	INIT => TxEnaAndLoadLutVal
	)
port map (
	I0	=>	IntTxByteCntOut(0),
	I1	=>	IntTxByteCntOut(1),
	I2	=>	EnaIn,
	I3	=>	IntEnaOut,
	O	=>	IntEnaAndLoad
);
--ByteLoad <= IntEnaAndLoad;		-- "ByteLoad" is enable of a Multiplexer (AND gate) to let new data pass through to the OSERDES.
TheEnaInFfOne : FDRE
	port map (
		D => IntEnaAndLoad,
		C => Clock0,
		CE => High,
		R => IntSyncReset,
		Q => IntEnaOut
	);
EnaOut <= IntEnaOut;			-- "EnaOut" enable of the OSERDES (pin OSERDES.OCE)
-- The signal EnaOut is the enable signal for the OSERDES, making sure everything starts at a rising edge of Clk0 and thus LxCLKOUT.
--
-------------------------------------------------------------------
-- Byte counter, Gray counter.
-------------------------------------------------------------------
GenTxByteCnt : for i in 1 downto 0 generate
--	attribute RLOC of TxByteIncrLut : label is TxByteRlocVal(i);
--	attribute RLOC of TxByteIncrFf : label is TxByteRlocVal(i);
	begin
	TxByteIncrLut : LUT4
	generic map (
		INIT => TxByteLutVal(i)
	)
	port map (
		I3	=> High,
		I2	=> IntEnaOut,
		I0 	=> IntTxByteCntOut(0),
		I1 	=> IntTxByteCntOut(1),
		O 	=> IntTxByteIncOut(i)
	);
	TxByteIncrFf : FDRE
	port map (
		D	=> IntTxByteIncOut(i),
		C	=> Clock0,
		CE	=> High,
		R	=> IntSyncReset,
		Q	=> IntTxByteCntOut(i)
	);
end generate;
--
TheByteCntTcLut : LUT4
	generic map (
		INIT => TxByteCntTcLutVal
	)
	port map (
		I3	=> High,
		I2	=> IntEnaOut,
		I1	=> IntTxByteCntOut(1),
		I0	=> IntTxByteCntOut(0),
		O 	=> IntByteClock
	);
TheByteCntTcFf : FDRE
	port map (
		D	=> IntByteClock,
		C	=> Clock0,
		CE	=> High,
		R	=> IntSyncReset,
		Q	=> ByteClock			-- "ByteClock" is the OSERDES.CLKDIV.
	);

-------------------------------------------------------------------
-- Quad word counter, Gray counter.
-------------------------------------------------------------------
--IntTxQwrdCntEna <= IntEnaOut and IntByteClock;
--
GenTxQwrdCnt : for i in 0 to 3 generate
--	attribute RLOC of TxQwrdIncrLut : label is TxQwrdRlocVal(i);
--	attribute RLOC of TxQwrdIncrFf : label is TxQwrdRlocVal(i);
	begin
	TxQwrdIncrLut : LUT4
	generic map (
		INIT => TxQwrdLutVal(i)
	)
	port map (
		I0 	=> IntTxQwrdCntOut(0),
		I1 	=> IntTxQwrdCntOut(1),
		I2 	=> IntTxQwrdCntOut(2),
		I3 	=> IntTxQwrdCntOut(3),
		O 	=> IntTxQwrdIncOut(i)
	);
	TxQwrdIncrFf : FDRE_1
	port map (
		D	=> IntTxQwrdIncOut(i),
		CE	=> IntByteClock,
		C	=> Clock0,
		R	=> IntSyncReset,
		Q	=> IntTxQwrdCntOut(i)
	);
end generate;

TheQwrdCntTcLut : LUT4
	generic map (
		INIT => TxQwrdCntTcLutVal
	)
	port map (
		I0 	=> IntTxQwrdIncOut(0),
		I1 	=> IntTxQwrdIncOut(1),
		I2 	=> IntTxQwrdIncOut(2),
		I3 	=> IntTxQwrdIncOut(3),
		O 	=> IntTxQwrdCntTc
	);
TheLxEnaLut : LUT2
	generic map (
		INIT => TxQwrdLxAckiLutVal
	)
	port map (
		I0	=> IntTxQwrdCntTc,
		I1	=> EnaIn,
		O	=> IntTxQwrdEnaLut
	);
TheQwrdCntTcFf : FDRE_1
	port map (
		D	=> IntTxQwrdEnaLut,
		CE	=> IntByteClock,
		C	=> Clock0,
		R	=> IntSyncReset,
		Q	=> IntSyncReset
	);
TheRstDlyFf : FD port Map (D => IntSyncReset, C => Clock0, Q => AsynReset);
--
end  TxBytAndQwrdCnt_arch ;