------------------------------------------------------------------------------------------------------------------------
-- Copyright (c) 1996-2003 Xilinx, Inc.
-- All Rights Reserved
------------------------------------------------------------------------------------------------------------------------
--
--  Disclaimer:
--              LIMITED WARRANTY AND DISCLAIMER.
--              These designs are provided to you "as is". Xilinx and its licensors make and you
--              receive no warranties or conditions, express, implied, statutory or otherwise, and
--              Xilinx specifically disclaims any implied warranties of merchantability, non-infringement,
--              or fitness for a particular purpose. Xilinx does not warrant that the functions contained
--              in these designs will meet your requirements, or that the operation of these designs will
--              be uninterrupted or error free, or that defects in the designs will be corrected.
--              Furthermore, Xilinx does not warrant or make any representations regarding use or the
--              results of the use of the designs in terms of correctness, accuracy, reliability, or otherwise.
--
--              LIMITATION OF LIABILITY.
--              In no event will Xilinx or its licensors be liable for any loss of data, lost profits, cost or
--              procurement of substitute goods or services, or for any special, incidental, consequential,
--              or indirect damages arising from the use or operation of the designs or accompanying
--              documentation, however caused and on any theory of liability. This limitation will apply
--              even if Xilinx has been advised of the possibility of such damage. This limitation shall
--              apply not-withstanding the failure of the essential purpose of any limited remedies herein.
--
--  Contact:    e-mail  hotline@xilinx.com        phone   + 1 800 255 7778
--   ____  ____
--  /   /\/   /
-- /___/  \  /   Vendor: Xilinx
-- \   \   \/    Version: 0.01
--  \   \        Filename: LinkPortRx.vhd
--  /   /        Date Last Modified:  
-- /___/   /\    Date Created: 23/08/05
-- \   \  /  \
--  \___\/\___\
-- 
-- Device: XC4VLX25-FF668-11
-- Author: Marc Defossez
-- Entity Name:  LinkPortRx
-- Purpose: Create a link port receiver styled the way the Virtex-IIpro Link port receiver was made.
--			This design does not make use of the Virtex-4 IO features as ISERDES and OSERDES.
-- Tools:	ISE7.1sp3
-- Limitations: none
--
-- Revision History:
--    Rev. 0.01
--
------------------------------------------------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_UNSIGNED.all;
-- Simulation libraries
	-- synopsys translate_off
	library UNISIM;
	use UNISIM.all;
	-- synopsys translate_on
-- end simulation libraries

-- Libraries for Synplicity
-- Needed for components as LUT, SRL which get an INIT parameter.
-- When this library is not mentioned Synplify_Pro fails during synthesis of
-- the hardware with an error about the INIT value.
-- library virtex2;
-- use virtex2.components.all;
-- This library is ignored by Mentor Precision.
-- Only a warning is generated.
-- End libraries for Synplicity

entity LinkPortRx is
    port (
    	-- System
    	RxClk			: in std_logic;		-- Clock at same rate of LxClk LinkPort clock.
    	RxRefClk		: in std_logic;		-- Clock (200 MHz) for the IDELAYCTRL element of the receiver.
    	SysClk			: in std_logic;		-- System clock reading the FIFO's.
    	SysRxFifoEna	: in std_logic;		-- System control of FIFO.
    	SysRxEna		: in std_logic; 	-- Receiver control.
    	SysRst			: in std_logic;		-- System reset, active low.
    	SysRxFlag 		: out std_logic_vector (2 downto 0); -- FIFO flags for the system.
    	SysRxData		: out std_logic_vector (127 downto 0); -- Received data
    	-- Interface
    	LxClkIn_p	: in std_logic;
    	LxClkIn_n	: in std_logic;
    	LxDati_p	: in std_logic_vector (3 downto 0);
    	LxDati_n	: in std_logic_vector (3 downto 0);
    	LxAcko		: out std_logic;
    	LxBcmpiN	: in std_logic
    );
end  LinkPortRx ;

architecture LinkPortRx_arch of LinkPortRx  is
------------------------------------------------------------------------------------------------------------------------
-- Component Instantiation
------------------------------------------------------------------------------------------------------------------------
component IBUFDS_LVDS_25 port (I, IB : in std_logic; O : out std_logic); end component;
component IBUFG_LVCMOS25 port (I : in std_logic; O : out std_logic);	 end component;
component IBUF_LVCMOS25	 port (I : in std_logic; O : out std_logic);	 end component;
component OBUF_LVCMOS25	 port (I : in std_logic; O : out std_logic);	 end component;
component BUFIO 		 port (I : in std_logic; O : out std_logic); 	 end component;
component BUFR			 generic (BUFR_DIVIDE : string);
						 port (O : out std_logic; CE, CLR, I : in  std_logic);  end component;
component FD			 port (D, C : in std_logic; Q : out std_logic);			end component;
component FDE			 port (D, C, CE : in std_logic; Q : out std_logic);			end component;
component FDC			 port (D, C, CLR : in std_logic; Q : out std_logic);	end component;
component RAM16X1D		 port (D, WE, WCLK, A0, A1, A2, A3, DPRA0, DPRA1, DPRA2, DPRA3 : in std_logic;
      	                 		 SPO, DPO : out std_logic); end component;
component RxBramMem
    port (
    	RxClk			: in std_logic;
        SysRst			: in std_logic;
        SysClk			: in std_logic;
        SysRxEna		: in std_logic;
        SysRxFifoEna	: in std_logic;
        SysRxData		: out std_logic_vector (127 downto 0);
        BramDataIn		: in std_logic_vector (7 downto 0);
        BramWe			: in std_logic_vector (3 downto 0);
        BramWrAdd		: in std_logic_vector (10 downto 0);
        BramRdAdd		: in std_logic_vector (8 downto 0)
    );
end component;
------------------------------------------------------------------------------------------------------------------------
-- Signal, Constants and Attributes Declarations
------------------------------------------------------------------------------------------------------------------------
-- Constants
type RlocValueX is array (0 to 15) of integer;
constant RlocVal_X : RlocValueX := (
	2,	-- LutRam P
	2, 
	2, 
	2,
	2,	-- LutRam N
	2,
	2,
	2,
	3,	-- RamFf P
	3,
	3,
	3,
	3,	-- RamFF N 
	3,
	3,
	3
);
type RlocValueY is array (0 to 15) of integer;
constant RlocVal_Y : RlocValueY := (
	10,	-- LutRam P
	5, 
	12, 
	3,
	11,	-- LutRam N
	4,
	13,
	2,
	10,	-- RamFf P
	5,
	12,
	3,
	11,	-- RamFF N 
	4,
	13,
	2
);
--
-- Signals
signal High	: std_logic;
signal Low	: std_logic;

signal IntLxAcko		: std_logic;
signal IntLxBcmpiN		: std_logic;
signal LxClkToBufIo 	: std_logic;
signal LxClkBufIoToBufr : std_logic;
signal IntLxClk			: std_logic;
signal IntLxDat3		: std_logic;
signal IntLxDat2		: std_logic;
signal IntLxDat1		: std_logic;
signal IntLxDat0		: std_logic;
signal IntLxDat			: std_logic_vector (3 downto 0);
signal IntLxClkNot		: std_Logic;

signal IntWrAddrp	: std_logic_vector (3 downto 0);	-- LUT RAM write address P data
signal IntWrAddrn 	: std_logic_vector (3 downto 0);	-- LUT RAM write address N data
signal IntRdAddr 	: std_logic_vector (3 downto 0);	-- LUT RAM read address
signal IntWrAddpFf 	: std_logic_vector (3 downto 0);	-- Sequencer registered P address
signal IntWrAddnFf 	: std_logic_vector (3 downto 0); -- Sequencer registered N address

signal IntLutRamDat : std_logic_vector (7 downto 0);	-- Data between Ram and FF.
signal IntFfDat  	: std_logic_vector (7 downto 0);		-- Data After FF.

signal IntWeGen 		: std_logic_vector (3 downto 0);
signal IntBramWe 		: std_logic_vector (3 downto 0);
signal IntWrAddrGen 	: std_logic_vector (10 downto 0);
signal IntBramWrAdd 	: std_logic_vector (10 downto 0);
signal IntBramRdAdd 	: std_logic_vector (8 downto 0);
signal IntBramWrAddFf 	: std_logic_vector (1 downto 0);
signal IntPreFlag 		: std_logic_vector (2 downto 0);
signal IntFifoFlag 		: std_logic_vector (2 downto 0);

-- Attributes
--attribute BUFR_DIVIDE : string;
--	attribute BUFR_DIVIDE of LxClk_Bufr	: label is "BYPASS";
attribute RLOC : string;
attribute KEEP : string;
	attribute KEEP of IntLxDat0 : signal is "TRUE";
	attribute KEEP of IntLxDat1 : signal is "TRUE";
	attribute KEEP of IntLxDat2 : signal is "TRUE";
	attribute KEEP of IntLxDat3 : signal is "TRUE";
attribute RLOC_ORIGIN : string;
	attribute RLOC_ORIGIN of BramMemory : label is "X0Y20";  -- 4VLX40-FF1148-11
------------------------------------------------------------------------------------------------------------------------
--
begin
--------------------------------------------------------------------
-- Stuff
--------------------------------------------------------------------
High <= '1';
Low <= '0';
--------------------------------------------------------------------
-- Inputs and Outputs
--------------------------------------------------------------------
LxAckO_Obuf 	: OBUF_LVCMOS25		port map (I => IntLxAcko, 	O => LxAcko);
LxBcmpiN_Ibuf 	: IBUF_LVCMOS25		port map (I => LxBcmpiN, 	O => IntLxBcmpiN);
--------
LxClk_Ibuf		: IBUFDS_LVDS_25	port map (I => LxClkIn_p,   	IB => LxClkIn_n,	O => LxClkToBufIo);
LxClk_BufIo		: BUFIO				port map (I => LxClkToBufIo, O => LxClkBufIoToBufr);
LxClk_Bufr		: BUFR				generic map (BUFR_DIVIDE => "BYPASS")
									port map (CE => High, CLR => Low, I => LxClkBufIoToBufr, O => IntLxClk);
--------
LxDat0_Ibuf	: IBUFDS_LVDS_25	port map (I => LxDati_p(0), IB => LxDati_n(0), O => IntLxDat0);
LxDat1_Ibuf	: IBUFDS_LVDS_25	port map (I => LxDati_p(1), IB => LxDati_n(1), O => IntLxDat1);
LxDat2_Ibuf	: IBUFDS_LVDS_25	port map (I => LxDati_p(2), IB => LxDati_n(2), O => IntLxDat2);
LxDat3_Ibuf	: IBUFDS_LVDS_25	port map (I => LxDati_p(3), IB => LxDati_n(3), O => IntLxDat3);

IntLxDat <= IntLxDat3 & IntLxDat2 & IntLxDat1 & IntLxDat0;
--------------------------------------------------------------------
-- Receiver LUT RAM
--------------------------------------------------------------------
GenTwo : for I in 0 to 3 generate
		attribute RLOC of LutRamP : label is "X" & integer'image(RlocVal_X(I)) & "Y" & integer'image(RlocVal_Y(I));
		attribute RLOC of LutRamN : label is "X" & integer'image(RlocVal_X(I+4)) & "Y" & integer'image(RlocVal_Y(I+4));
		attribute RLOC of RamFfP : label is "X" & integer'image(RlocVal_X(I+8)) & "Y" & integer'image(RlocVal_Y(I+8));
		attribute RLOC of RamFfN : label is "X" & integer'image(RlocVal_X(I+12)) & "Y" & integer'image(RlocVal_Y(I+12));
	begin
	LutRamP : RAM16X1D
		port map (
			D		=> IntLxDat(I),
			WE		=> High,
			WCLK	=> IntLxClk,
			A0		=> IntWrAddrp(0),
			A1		=> IntWrAddrp(1),
		    A2		=> IntWrAddrp(2),
		    A3		=> IntWrAddrp(3),
		    DPRA0	=> IntRdAddr(0),
		    DPRA1	=> IntRdAddr(1),
		    DPRA2	=> IntRdAddr(2),
		    DPRA3	=> IntRdAddr(3),
		    SPO 	=> open,
		    DPO 	=> IntLutRamDat(I+I)
		    );
	RamFfP : FDE port map (D => IntLutRamDat(I+I), C => RxClk, CE => SysRxEna, Q => IntFfDat(I+I));
	LutRamN : RAM16X1D
		port map (
			D		=> IntLxDat(I),
			WE		=> High,
			WCLK	=> IntLxClk,
			A0		=> IntWrAddrn(0),
			A1		=> IntWrAddrn(1),
		    A2		=> IntWrAddrn(2),
		    A3		=> IntWrAddrn(3),
		    DPRA0	=> IntRdAddr(0),
		    DPRA1	=> IntRdAddr(1),
		    DPRA2	=> IntRdAddr(2),
		    DPRA3	=> IntRdAddr(3),
		    SPO 	=> open,
		    DPO 	=> IntLutRamDat(I+(I+1))
		    );
	RamFfN : FDE port map (D => IntLutRamDat(I+(I+1)), C => RxClk, CE => SysRxEna, Q => IntFfDat(I+(I+1)));
	end generate;
--------------------------------------------------------------------
-- LUT RAM address counter / sequencer
--------------------------------------------------------------------
IntLxClkNot <= not IntLxClk;

IntWrAddpFf <=	"0000" when IntWrAddrp = "1111" else
	      		"0001" when IntWrAddrp = "0000" else
				"0010" when IntWrAddrp = "0001" else
				"0011" when IntWrAddrp = "0010" else
				"0100" when IntWrAddrp = "0011" else
				"0101" when IntWrAddrp = "0100" else
				"0110" when IntWrAddrp = "0101" else
				"0111" when IntWrAddrp = "0110" else
				"1000" when IntWrAddrp = "0111" else
				"1001" when IntWrAddrp = "1000" else
				"1010" when IntWrAddrp = "1001" else
				"1011" when IntWrAddrp = "1010" else
				"1100" when IntWrAddrp = "1011" else
				"1101" when IntWrAddrp = "1100" else
				"1110" when IntWrAddrp = "1101" else
				"1111";
GenFor : for I in 0 to 3 generate
	WrAddpFf : FDC
		port map (
			D => IntWrAddpFf(I), C => IntLxClk, CLR => SysRst, Q => IntWrAddrp(I)
			);
	end generate;
GenFiv : for I in 0 to 3 generate
	IntWrAddnFf(I) <= not SysRst and IntWrAddrp(I);
	WrAddnFf : FD
		port map (
			D => IntWrAddnFf(I), C => IntLxClkNot, Q => IntWrAddrn(I)
			);
	end generate;
--------------------------------------------------------------------
-- BRAM address logic
--------------------------------------------------------------------
BramWrAddrGen : process (RxClk, SysRst)
begin
if (SysRst = '1') then
	IntRdAddr <= X"0";
	IntWrAddrGen <= "00000000000";
	IntBramWrAdd <= "00000000000";
	IntWeGen <= X"1";
	IntBramWe <= X"0";
	IntLxAcko <= '0';
elsif (RxClk'event and RxClk = '1') then
	if (SysRxEna = '0') then
		IntBramWe <= X"0";
	else
		IntBramWe <= IntWeGen;
	end if;
	
	IntBramWrAdd <= IntWrAddrGen (10 downto 2) & IntRdAddr (1 downto 0);
	IntLxAcko <= SysRxEna and IntFifoFlag(2);

-- LutRam Read address logic.
	if (SysRxEna = '0') then
		IntRdAddr <= "0000";
	elsif (SysRxEna = '1' and IntWrAddrp > "1000" and IntRdAddr = "0000") then
		IntRdAddr <= IntRdAddr + 1;
	elsif (IntRdAddr /= "0000") then
		IntRdAddr <= IntRdAddr + 1;
	end if;
--
	if (IntRdAddr = "1111") then
		IntWeGen <= "0001";
		IntWrAddrGen (5 downto 2) <= IntWrAddrGen (5 downto 2) + 1;
		if (IntWrAddrGen (5 downto 2) = "1111") then
			case IntWrAddrGen (8 downto 6) is
				when "000" => IntWrAddrGen(8 downto 6) <= "001";
				when "001" => IntWrAddrGen(8 downto 6) <= "011";
				when "011" => IntWrAddrGen(8 downto 6) <= "010";
				when "010" => IntWrAddrGen(8 downto 6) <= "110";
				when "110" => IntWrAddrGen(8 downto 6) <= "111";
				when "111" => IntWrAddrGen(8 downto 6) <= "101";
				when "101" => IntWrAddrGen(8 downto 6) <= "100";
--				when "100" => IntWrAddrGen(8 downto 6) <= "000";
				when others => IntWrAddrGen(8 downto 6) <= "000";
			end case;
		end if;
		if (IntWrAddrGen (8 downto 6) = "111" and IntWrAddrGen (5 downto 2) = "1111") then
			case IntWrAddrGen (10 downto 9) is
				when "00" => IntWrAddrGen(10 downto 9) <= "01";
				when "01" => IntWrAddrGen(10 downto 9) <= "11";
				when "11" => IntWrAddrGen(10 downto 9) <= "10";
--				when "10" => IntWrAddrGen(10 downto 9) <= "00";
				when others => IntWrAddrGen(10 downto 9) <= "00";
			end case;
		end if;
	elsif (IntRdAddr = "0011") then
		IntWeGen <= "0010";
	elsif (IntRdAddr = "0111") then
		IntWeGen <= "0100";
	elsif (IntRdAddr = "1011") then
		IntWeGen <= "1000";
	end if;
end if;

end process;

BramRdAddrGen : process (SysClk, SysRst)
begin
if (SysRst = '1') then
	IntBramRdAdd <= "000000000" ;
elsif (SysClk'event and SysClk = '1') then
	if (SysRxFifoEna = '1') then
		IntBramRdAdd(6 downto 0) <= IntBramRdAdd(6 downto 0) + 1 ;
		if (IntBramRdAdd(6 downto 0) = "1111111") then
			case IntBramRdAdd(8 downto 7) is
				when "00" => IntBramRdAdd(8 downto 7) <= "01" ;
				when "01" => IntBramRdAdd(8 downto 7) <= "11" ;
				when "11" => IntBramRdAdd(8 downto 7) <= "10" ;
				when others => IntBramRdAdd(8 downto 7) <= "00" ;
			end case ;
		end if ;
	end if ;
end if ;
end process ;
--------------------------------------------------------------------
-- BRAM components
--------------------------------------------------------------------
BramMemory : RxBramMem
    port map (
    	RxClk			=> RxClk,
        SysRst			=> SysRst,
        SysClk			=> SysClk,
        SysRxEna		=> SysRxEna,
        SysRxFifoEna	=> SysRxFifoEna,
        SysRxData		=> SysRxData,
        BramDataIn		=> IntFfDat,
        BramWe			=> IntBramWe,
        BramWrAdd		=> IntBramWrAdd,
        BramRdAdd		=> IntBramRdAdd
    );
-------------------------------------------------------------------
-- FIFO flags
--------------------------------------------------------------------
WrAddrFf0 : FDC port map (D => IntBramWrAdd(9), C => SysClk, CLR => SysRst, Q => IntBramWrAddFf(0));
WrAddrFf1 : FDC port map (D => IntBramWrAdd(10), C => SysClk, CLR => SysRst, Q => IntBramWrAddFf(1));

IntPreFlag(2) <= '1' when IntBramRdAdd(8 downto 7) = "00" and IntBramWrAddFf = "10" else -- Buffer is between half full and full.
				 '1' when IntBramRdAdd(8 downto 7) = "01" and IntBramWrAddFf = "00" else
				 '1' when IntBramRdAdd(8 downto 7) = "11" and IntBramWrAddFf = "01" else
				 '1' when IntBramRdAdd(8 downto 7) = "10" and IntBramWrAddFf = "11" else
				 '0' ;
FlagFf2 : FDC port map (D => IntPreFlag(2), C => SysClk, CLR => SysRst, Q => IntFifoFlag(2));
IntPreFlag(1) <= '1' when IntBramRdAdd(8 downto 7) = "00" and IntBramWrAddFf = "11" else -- Buffer is between a quarter full and 
				 '1' when IntBramRdAdd(8 downto 7) = "01" and IntBramWrAddFf = "10" else	 -- three quarters full
				 '1' when IntBramRdAdd(8 downto 7) = "11" and IntBramWrAddFf = "00" else
				 '1' when IntBramRdAdd(8 downto 7) = "10" and IntBramWrAddFf = "01" else
				 '0' ;
FlagFf1 : FDC port map (D => IntPreFlag(1), C => SysClk, CLR => SysRst, Q => IntFifoFlag(1));
IntPreFlag(0) <= '1' when IntBramRdAdd(8 downto 7) = "00" and IntBramWrAddFf = "01" else -- Buffer is between nearly empty and
				 '1' when IntBramRdAdd(8 downto 7) = "01" and IntBramWrAddFf = "11" else	 -- a quarter full
				 '1' when IntBramRdAdd(8 downto 7) = "11" and IntBramWrAddFf = "10" else
				 '1' when IntBramRdAdd(8 downto 7) = "10" and IntBramWrAddFf = "00" else
				 '0' ;
FlagFf0 : FDC port map (D => IntPreFlag(0), C => SysClk, CLR => SysRst, Q => IntFifoFlag(0));

SysRxFlag <= IntFifoFlag;
--------------------------------------------------------------------
--
end  LinkPortRx_arch ;