------------------------------------------------------------------------------------------------------------------------
-- Copyright (c) 1996-2003 Xilinx, Inc.
-- All Rights Reserved
------------------------------------------------------------------------------------------------------------------------
--
--  Disclaimer:
--              LIMITED WARRANTY AND DISCLAIMER.
--              These designs are provided to you "as is". Xilinx and its licensors make and you
--              receive no warranties or conditions, express, implied, statutory or otherwise, and
--              Xilinx specifically disclaims any implied warranties of merchantability, non-infringement,
--              or fitness for a particular purpose. Xilinx does not warrant that the functions contained
--              in these designs will meet your requirements, or that the operation of these designs will
--              be uninterrupted or error free, or that defects in the designs will be corrected.
--              Furthermore, Xilinx does not warrant or make any representations regarding use or the
--              results of the use of the designs in terms of correctness, accuracy, reliability, or otherwise.
--
--              LIMITATION OF LIABILITY.
--              In no event will Xilinx or its licensors be liable for any loss of data, lost profits, cost or
--              procurement of substitute goods or services, or for any special, incidental, consequential,
--              or indirect damages arising from the use or operation of the designs or accompanying
--              documentation, however caused and on any theory of liability. This limitation will apply
--              even if Xilinx has been advised of the possibility of such damage. This limitation shall
--              apply not-withstanding the failure of the essential purpose of any limited remedies herein.
--
--  Contact:    e-mail  hotline@xilinx.com        phone   + 1 800 255 7778
--   ____  ____
--  /   /\/   /
-- /___/  \  /   Vendor: Xilinx
-- \   \   \/    Version: 0.01
--  \   \        Filename: LinkPortTx.vhd
--  /   /        Date Last Modified:  
-- /___/   /\    Date Created: 30/08/05
-- \   \  /  \
--  \___\/\___\
-- 
-- Device: Virtex-4
-- Author: Marc Defossez
-- Entity Name:  LinkPortTx
-- Purpose: Create a LinkPort transmitter design
-- Tools: ISE7.1.03
-- Limitations: none
--
-- Revision History:
--    Rev. 
--
------------------------------------------------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_UNSIGNED.all;
-- Simulation libraries
	-- synopsys translate_off
	library UNISIM;
	use UNISIM.all;
	-- synopsys translate_on
-- end simulation libraries

-- Libraries for Synplicity
-- Needed for components as LUT, SRL which get an INIT parameter.
-- When this library is not mentioned Synplify_Pro fails during synthesis of
-- the hardware with an error about the INIT value.
-- library virtex2;
-- use virtex2.components.all;
-- This library is ignored by Mentor Precision.
-- Only a warning is generated.
-- End libraries for Synplicity

entity LinkPortTx is
    port (
    	-- System
        TxClk		: in std_logic;			-- Used in the whole design.
        TxClk90		: in std_logic;			-- Used by the LxClkOut.
        SysClk		: in std_logic;
        SysRst		: in std_logic;
        SysTxFifoEna : in std_logic;		-- Enable of the TX Bram write side, Fill the RAM side.
        SysTxEna	: in std_logic;			-- Enable of the TX Bram read side, tranmitter side.
        SysTxFifoFull : out std_logic;
        SysTxFifoEmpty	: out std_logic;		-- Flag of empty FIFO.
        SysLxAcki	: out std_logic;		-- Passing of LxAcki signal.
        SysTxData	: in std_logic_vector (127 downto 0);
        -- Interface
        LxClkOut_p	: out std_logic;
        LxClkOut_n	: out std_logic;
        LxDato_p	: out std_logic_vector (3 downto 0);
        LxDato_n	: out std_logic_vector (3 downto 0);
        LxAcki		: in std_logic;
        LxBcmpoN	: out std_logic
    );
end  LinkPortTx ;

architecture LinkPortTx_arch of LinkPortTx  is
------------------------------------------------------------------------------------------------------------------------
-- Component Instantiation
------------------------------------------------------------------------------------------------------------------------
component IBUF_LVCMOS25		port (I : in std_logic; O : out std_logic);			end component;
component OBUF_LVCMOS25_S_12		port (I : in std_logic; O : out std_logic);			end component;
component FD				port (C, D : in std_logic; Q : out std_logic);		end component;
component FDC				port (C, D, CLR : in std_logic; Q : out std_logic); end component;

component TxBramMem
port (
	RamPortaEna		: in std_logic;
	SysRst			: in std_logic;
	TxClk			: in std_logic;
	RamRdAddr		: in std_logic_vector (11 downto 0);
	RamDataOut		: out std_logic_vector (15 downto 0);
	SysTxData		: in std_logic_vector (127 downto 0);
	SysTxFifoEna	: in std_logic;
	SysClk			: in std_logic;
	RamWrAddr		: in std_logic_vector (8 downto 0)
);
end component;

component OdataSerdes
port (
    	-- System
        DatSrdsClk		: in std_logic;		-- High speed clock from DCM
        DatSrdsDatIn	: in std_logic_vector (15 downto 0);	-- Data from the back-end system
        DatSrdsRst		: in std_logic;		-- System Reset
        DatSrdsMuxSel	: in std_logic;		-- Multiplexer select input
        -- Interface
        LxDato_p	: out std_logic_vector (3 downto 0);	-- DDR FPGA output
        LxDato_n	: out std_logic_vector (3 downto 0)	-- DDR FPGA output
	);
end component;

component OclockSerdes
port (
    	-- System
        ClkSrdsClk		: in std_logic;		-- High speed clock from DCM = TxClk90
        ClkSrdsRst		: in std_logic;		-- System Reset
        ClkSrdsMuxSel	: in std_logic; 	-- System Output Enable
        ClkSrdsDataIn	: in std_logic_vector (3 downto 0);
        -- Interface
        LxClkOut_p	: out std_logic;	-- DDR FPGA output
        LxClkOut_n	: out std_logic		-- DDR FPGA output
	);
end component;

component Mux2To1EnaRst
    port (
    	MuxClock	: in std_logic;	-- clock
    	MuxReset	: in std_logic;	-- reset
		MuxDatIn0 	: in std_logic;	-- data input 0
		MuxDatIn1 	: in std_logic;	-- data input 1
		MuxLutEna	: in std_logic;	-- mux LUT enable
		MuxFfEna	: in std_logic;	-- mux FF enable
		MuxSelect	: in std_logic;	-- mux select
		MuxCbDatOut	: out std_logic;	-- combinatorial output
		MuxFfDatOut	: out std_logic	-- registered output
    );
end component;
------------------------------------------------------------------------------------------------------------------------
-- Signal, Constants and Attributes Declarations
------------------------------------------------------------------------------------------------------------------------
-- Constants
type RlocValueX is array (0 to 15) of integer;
constant RlocVal_X : RlocValueX := (
	4,	--
	4,
	4,
	4,	--
	4,
	4,
	4,
	4,	--
	4, 
	4, 
	4, 
	4,	--
	4, 
	4,
	4,
	4
);
type RlocValueY is array (0 to 15) of integer;
constant RlocVal_Y : RlocValueY := (
	5,	-- 
	10, 
	3, 
	12,	--
	4, 
	11,
	2,
	13,	--
	5, 
	10, 
	3, 
	12,	--
	4, 
	11,
	2,
	13
);
-- Signals
signal High				: std_logic;
signal Low 				: std_logic;
signal LowNibble 		: std_logic_vector (3 downto 0);
signal LowDbyte			: std_logic_vector (15 downto 0);
signal ClockNibble		: std_logic_vector (3 downto 0);
signal IntLxAcki		: std_logic;

signal IntNotDatProceed	: std_logic;
signal IntDatProceed	: std_logic;
signal IntNotClkProceed	: std_logic;
signal IntClkProceed	: std_logic;

signal IntNotPrcdAndTxEna	: std_logic;
signal IntTxEnaOneAndTre	: std_logic;
signal IntRamDatOut		: std_logic_vector (15 downto 0);
signal IntMuxFfDatOut	: std_logic_vector (15 downto 0);
signal IntMuxFfClkOut	: std_logic_vector (3 downto 0);
signal IntSysTxEnaOne	: std_logic;
signal IntSysTxEnaTwo	: std_logic;
signal IntSysTxEnaTre	: std_logic;

signal IntRamWrAddr		: std_logic_vector (8 downto 0);
signal IntTxDataOut		: std_logic_vector (15 downto 0);
signal IntRamRdAddr		: std_logic_vector (11 downto 0);
signal IntEmptyA		: std_logic;
signal IntEmptyB		: std_logic;
signal IntRamRdAddrFix 	: std_logic_vector (8 downto 0);
-- Attributes
attribute RLOC : string;
--	attribute RLOC of DatPrcdFf : label is "X6Y7";
--	attribute RLOC of ClkPrcdFf : label is "X6Y14";
attribute RLOC_ORIGIN : string;
	attribute RLOC_ORIGIN of TxBram : label is "X0Y16";  -- 4VLX40-FF1148-11
	attribute RLOC_ORIGIN of DataSerdes : label is "X0Y136";  -- 4VLX40-FF1148-11
	attribute RLOC_ORIGIN of ClockSerdes : label is "X0Y136";  -- 4VLX40-FF1148-11
attribute KEEP : string;
	attribute KEEP of IntMuxFfClkOut : signal is "TRUE";
------------------------------------------------------------------------------------------------------------------------
--
begin

--------------------------------------------------------------------
-- Stuff
--------------------------------------------------------------------
High <= '1';
Low <= '0';

-- Divide the TxClk by two and create an enable signal.
-- The proceed is due the Tcko and the routing shifted to the TxClk.
-- Shift tested between 300ps and 1.5 ns.
IntNotDatProceed <= not IntDatProceed;
DatPrcdFf : FDC
	port map (
		D	=> IntNotDatProceed,
		C	=> TxClk,
		CLR	=> SysRst,
		Q	=> IntDatProceed
	);
IntNotClkProceed <= not IntClkProceed;
ClkPrcdFf : FDC
	port map (
		D	=> IntNotClkProceed,
		C	=> TxClk90,
		CLR	=> SysRst,
		Q	=> IntClkProceed
	);

LxAcki_IBUF : IBUF_LVCMOS25		port map (I => LxAcki, O => IntLxAcki);
SysLxAcki <= IntLxAcki;

LxBcmpoN_OBUF : OBUF_LVCMOS25_S_12	port map (I => High, O => LxBcmpoN);
--------------------------------------------------------------------
-- Transmitter Block-RAM 
--------------------------------------------------------------------
TxBram : TxBramMem
port map (
	RamPortaEna		=> IntNotPrcdAndTxEna,
	SysRst			=> SysRst,
	TxClk			=> TxClk,
	RamRdAddr		=> IntRamRdAddr,
	RamDataOut		=> IntRamDatOut,
	SysTxData		=> SysTxData,
	SysTxFifoEna	=> SysTxFifoEna,
	SysClk			=> SysClk,
	RamWrAddr		=> IntRamWrAddr
);
--------------------------------------------------------------------
-- Muxes and registers
--------------------------------------------------------------------
LowDbyte <= "0000000000000000";

GenOne : for I in 0 to 15 generate
--	attribute RLOC of TxDatMux : label is "X" & integer'image(RlocVal_X(I)) & "Y" & integer'image(RlocVal_Y(I));
	begin
	TxDatMux : Mux2To1EnaRst
	    port map (
	    	MuxClock	=> TxClk,
	    	MuxReset	=> SysRst,
			MuxDatIn0 	=> LowDbyte(I),
			MuxDatIn1 	=> IntRamDatOut(I),
			MuxLutEna	=> High,
			MuxFfEna	=> IntNotDatProceed,
			MuxSelect	=> IntTxEnaOneAndTre,
			MuxCbDatOut	=> open,
			MuxFfDatOut	=> IntMuxFfDatOut(I)
	    );
end generate;

LowNibble <= "0000";
ClockNibble <= "0101";

GenTwo : for I in 0 to 3 generate
	TxClkMux : Mux2To1EnaRst
	    port map (
	    	MuxClock	=> TxClk90,
	    	MuxReset	=> SysRst,
			MuxDatIn0 	=> LowNibble(I),
			MuxDatIn1 	=> ClockNibble(I),
			MuxLutEna	=> High,
			MuxFfEna	=> IntNotClkProceed,
			MuxSelect	=> IntTxEnaOneAndTre,
			MuxCbDatOut	=> open,
			MuxFfDatOut	=> IntMuxFfClkOut(I)
	    );
end generate;
--------------------------------------------------------------------
-- Clock and Data serdes
--------------------------------------------------------------------
DataSerdes : OdataSerdes
	port map (
		-- System
	    DatSrdsClk		=> TxClk,
	    DatSrdsDatIn	=> IntMuxFfDatOut,
	    DatSrdsRst		=> SysRst,
	    DatSrdsMuxSel	=> IntDatProceed,
	    -- Interface
	    LxDato_p	=> LxDato_p,
	    LxDato_n	=> LxDato_n
	);

ClockSerdes : OclockSerdes
	port map (
		-- System
	    ClkSrdsClk		=> TxClk90,
	    ClkSrdsRst		=> SysRst,
	    ClkSrdsMuxSel	=> IntClkProceed,
	    ClkSrdsDataIn	=> IntMuxFfClkOut,
	    -- Interface
	    LxClkOut_p	=> LxClkOut_p,
	    LxClkOut_n	=> LxClkOut_n
	);
--------------------------------------------------------------------
-- Transmitter Block-RAM B-port side
--------------------------------------------------------------------
WrAddrCnt : process (SysClk, SysRst)
begin
if (SysRst = '1') then
	IntRamWrAddr <= (others => '0');
	SysTxFifoFull <= '0';
elsif (SysClk'event and SysClk = '1') then
	if (SysTxFifoEna = '1') then
		if (IntRamWrAddr = "111111110") then
			SysTxFifoFull <= '1';
		else
			SysTxFifoFull <= '0';
		end if;
		
		IntRamWrAddr (2 downto 0) <= IntRamWrAddr (2 downto 0) + '1';
		if (IntRamWrAddr (2 downto 0) = "11") then
			case IntRamWrAddr (5 downto 3) is
				when "000" => IntRamWrAddr (5 downto 3) <= "001";
				when "001" => IntRamWrAddr (5 downto 3) <= "011";
				when "011" => IntRamWrAddr (5 downto 3) <= "010";
				when "010" => IntRamWrAddr (5 downto 3) <= "110";
				when "110" => IntRamWrAddr (5 downto 3) <= "111";
				when "111" => IntRamWrAddr (5 downto 3) <= "101";
				when "101" => IntRamWrAddr (5 downto 3) <= "100";
--				when "100" => IntRamWrAddr (5 downto 3) <= "000";
				when others  => IntRamWrAddr (5 downto 3) <= "000";
			end case;
		end if;
		if (IntRamWrAddr (5 downto 3) = "111" and IntRamWrAddr (2 downto 0) = "11") then
			case IntRamWrAddr (8 downto 6) is
				when "000" => IntRamWrAddr (8 downto 6) <= "001";
				when "001" => IntRamWrAddr (8 downto 6) <= "011";
				when "011" => IntRamWrAddr (8 downto 6) <= "010";
				when "010" => IntRamWrAddr (8 downto 6) <= "110";
				when "110" => IntRamWrAddr (8 downto 6) <= "111";
				when "111" => IntRamWrAddr (8 downto 6) <= "101";
				when "101" => IntRamWrAddr (8 downto 6) <= "100";
--				when "100" => IntRamWrAddr (8 downto 6) <= "000";
				when others  => IntRamWrAddr (8 downto 6) <= "000";
			end case;
		end if;
	end if;
end if;
end process;
-----------------------------------------------1---------------------
-- Transmitter Block-RAM A-port side
--------------------------------------------------------------------
IntTxDataOut <= IntRamDatOut;

RdAddrCnt : process (TxClk, SysRst, IntNotPrcdAndTxEna)		-- Transmit state machine
begin
if (SysRst = '1') then
	IntRamRdAddr <= "000000000000";
elsif (TxClk'event and TxClk = '1') then
	if (IntNotPrcdAndTxEna = '1') then		-- Counter enable
		IntRamRdAddr (3 downto 0) <= IntRamRdAddr (3 downto 0) + '1';
		if (IntRamRdAddr (3 downto 0) = "1111") then
			case IntRamRdAddr (7 downto 4) is
					when "0000" => IntRamRdAddr (7 downto 4) <= "0001";
					when "0001" => IntRamRdAddr (7 downto 4) <= "0011";
					when "0011" => IntRamRdAddr (7 downto 4) <= "0010";
					when "0010" => IntRamRdAddr (7 downto 4) <= "0110";
					when "0110" => IntRamRdAddr (7 downto 4) <= "0111";
					when "0111" => IntRamRdAddr (7 downto 4) <= "0101";
					when "0101" => IntRamRdAddr (7 downto 4) <= "0100";
					when "0100" => IntRamRdAddr (7 downto 4) <= "1100";
					when "1100" => IntRamRdAddr (7 downto 4) <= "1101";
					when "1101" => IntRamRdAddr (7 downto 4) <= "1111";
					when "1111" => IntRamRdAddr (7 downto 4) <= "1110";
					when "1110" => IntRamRdAddr (7 downto 4) <= "1010";
					when "1010" => IntRamRdAddr (7 downto 4) <= "1011";
					when "1011" => IntRamRdAddr (7 downto 4) <= "1001";
					when "1001" => IntRamRdAddr (7 downto 4) <= "1000";
--					when "1000" => IntRamRdAddr (7 downto 4) <= "0000";
					when others  => IntRamRdAddr (7 downto 4) <= "0000";
				end case;
		end if;
		if (IntRamRdAddr (7 downto 4) = "1111" and IntRamRdAddr (3 downto 0) = "1111") then
			case IntRamRdAddr (11 downto 8) is
					when "0000" => IntRamRdAddr (11 downto 8) <= "0001";
					when "0001" => IntRamRdAddr (11 downto 8) <= "0011";
					when "0011" => IntRamRdAddr (11 downto 8) <= "0010";
					when "0010" => IntRamRdAddr (11 downto 8) <= "0110";
					when "0110" => IntRamRdAddr (11 downto 8) <= "0111";
					when "0111" => IntRamRdAddr (11 downto 8) <= "0101";
					when "0101" => IntRamRdAddr (11 downto 8) <= "0100";
					when "0100" => IntRamRdAddr (11 downto 8) <= "1100";
					when "1100" => IntRamRdAddr (11 downto 8) <= "1101";
					when "1101" => IntRamRdAddr (11 downto 8) <= "1111";
					when "1111" => IntRamRdAddr (11 downto 8) <= "1110";
					when "1110" => IntRamRdAddr (11 downto 8) <= "1010";
					when "1010" => IntRamRdAddr (11 downto 8) <= "1011";
					when "1011" => IntRamRdAddr (11 downto 8) <= "1001";
					when "1001" => IntRamRdAddr (11 downto 8) <= "1000";
--					when "1000" => IntRamRdAddr (11 downto 8) <= "0000";
					when others  => IntRamRdAddr (11 downto 8) <= "0000";
			end case;
		end if;
	end if;
end if;
end process;
--------------------------------------------------------------------
-- Transmitter control
--------------------------------------------------------------------
EmptyCtrl : process (TxClk, SysRst, IntDatProceed)							-- read RAM process
begin
	if (SysRst = '1') then
		SysTxFifoEmpty <= '1';
		IntEmptyA <= '1';
		IntEmptyB <= '1';
		IntRamRdAddrFix <= (others => '0');
	elsif (TxClk'event and TxClk = '1') then
		IntRamRdAddrFix <= IntRamRdAddr(11 downto 3) + ("00000000" & (IntRamRdAddr(2) or IntRamRdAddr(1) or IntRamRdAddr(0)));
		IntEmptyB <= IntEmptyA;
		if (IntDatProceed = '1') then
			if (IntRamWrAddr = IntRamRdAddrFix) then
				IntEmptyA <= '1';
			else
				IntEmptyA <= '0';
			end if ;
		end if;
		SysTxFifoEmpty <= IntEmptyA and IntEmptyB;
	end if;
end process;

OutCtrl : process (TxClk, SysRst, SysTxEna)
begin
	if (SysRst = '1') then
		IntSysTxEnaOne <= '0';
		IntSysTxEnaTwo <= '0';
		IntSysTxEnaTre <= '0';
	elsif (TxClk'event and TxClk = '1') then
		IntSysTxEnaOne <= SysTxEna;
		IntSysTxEnaTwo <= IntSysTxEnaOne;
		IntSysTxEnaTre <= IntSysTxEnaTwo;
	end if;
end process;
			
IntNotPrcdAndTxEna <= not IntDatProceed and SysTxEna;
IntTxEnaOneAndTre <= IntSysTxEnaOne and IntSysTxEnaTre;

--
--------------------------------------------------------------------
end  LinkPortTx_arch ;