------------------------------------------------------------------------------------------------------------------------
-- Copyright (c) 1996-2003 Xilinx, Inc.
-- All Rights Reserved
------------------------------------------------------------------------------------------------------------------------
--
--  Disclaimer:
--              LIMITED WARRANTY AND DISCLAIMER.
--              These designs are provided to you "as is". Xilinx and its licensors make and you
--              receive no warranties or conditions, express, implied, statutory or otherwise, and
--              Xilinx specifically disclaims any implied warranties of merchantability, non-infringement,
--              or fitness for a particular purpose. Xilinx does not warrant that the functions contained
--              in these designs will meet your requirements, or that the operation of these designs will
--              be uninterrupted or error free, or that defects in the designs will be corrected.
--              Furthermore, Xilinx does not warrant or make any representations regarding use or the
--              results of the use of the designs in terms of correctness, accuracy, reliability, or otherwise.
--
--              LIMITATION OF LIABILITY.
--              In no event will Xilinx or its licensors be liable for any loss of data, lost profits, cost or
--              procurement of substitute goods or services, or for any special, incidental, consequential,
--              or indirect damages arising from the use or operation of the designs or accompanying
--              documentation, however caused and on any theory of liability. This limitation will apply
--              even if Xilinx has been advised of the possibility of such damage. This limitation shall
--              apply not-withstanding the failure of the essential purpose of any limited remedies herein.
--
--  Contact:    e-mail  hotline@xilinx.com        phone   + 1 800 255 7778
--   ____  ____
--  /   /\/   /
-- /___/  \  /   Vendor: Xilinx
-- \   \   \/    Version: 0.01
--  \   \        Filename: Mux2To1EnaRst.vhd
--  /   /        Date Last Modified:  
-- /___/   /\    Date Created: 23/08/05
-- \   \  /  \
--  \___\/\___\
-- 
-- Device: Virtex-IIpro, Virtex-4
-- Author: Marc Defossez
-- Entity Name:  Mux2To1EnaRst
-- Purpose: Multiplexer and register in one slice, Multipler with or without register
-- Tools: ISE 7.1.03
-- Limitations: none
--
-- Revision History:
--    Rev. 
--
------------------------------------------------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_UNSIGNED.all;
-- Simulation libraries
	-- synopsys translate_off
	library UNISIM;
	use UNISIM.all;
	-- synopsys translate_on
-- end simulation libraries

-- Libraries for Synplicity
-- Needed for components as LUT, SRL which get an INIT parameter.
-- When this library is not mentioned Synplify_Pro fails during synthesis of
-- the hardware with an error about the INIT value.
-- library virtex2;
-- use virtex2.components.all;
-- This library is ignored by Mentor Precision.
-- Only a warning is generated.
-- End libraries for Synplicity

entity Mux2To1EnaRst is
    port (
    	MuxClock	: in std_logic;	-- clock
    	MuxReset	: in std_logic;	-- reset
		MuxDatIn0 	: in std_logic;	-- data input 0
		MuxDatIn1 	: in std_logic;	-- data input 1
		MuxLutEna	: in std_logic;	-- mux LUT enable
		MuxFfEna	: in std_logic;	-- mux FF enable
		MuxSelect	: in std_logic;	-- mux select
		MuxCbDatOut	: out std_logic;	-- combinatorial output
		MuxFfDatOut	: out std_logic	-- registered output
    );
end  Mux2To1EnaRst ;

architecture Mux2To1EnaRst_arch of Mux2To1EnaRst  is
------------------------------------------------------------------------------------------------------------------------
-- Component Instantiation
------------------------------------------------------------------------------------------------------------------------
component LUT4
	generic (
		INIT 	: bit_vector
		);
	port (
		I0    	: in std_logic;	-- DataIn 0
		I1    	: in std_logic; -- DataIn 1
		I2    	: in std_logic; -- MuxSel
		I3    	: in std_logic; -- MuxEna
		O    	: out std_logic -- MuxOut
		);
end component ;

component FDCE
	port (
		D : in std_logic;
		C : in std_logic;
		CE : in std_logic;
		CLR : in std_logic;
		Q : out std_logic
		);
end component ;
------------------------------------------------------------------------------------------------------------------------
-- Signal, Constants and Attributes Declarations
------------------------------------------------------------------------------------------------------------------------
-- Constants
constant InitVector : bit_vector := X"CA00";	-- LUT equation
-- Signals
signal IntLutToFf : std_logic;
-- Attributes
attribute RLOC : string ;
	attribute RLOC of Mux : label is "X0Y0";
	attribute RLOC of Reg : label is "X0Y0";
------------------------------------------------------------------------------------------------------------------------
--
begin

Mux : LUT4  
	generic map (
		INIT => InitVector
	) 
	port map (
		I0 => MuxDatIn0,
		I1 => MuxDatIn1,
		I2 => MuxSelect,
		I3 => MuxLutEna,
		O => IntLutToFf
		);
MuxCbDatOut <= IntLutToFf;
Reg : FDCE
	port map (
		D => IntLutToFf,
		C => MuxClock,
		CE => MuxFfEna,
		CLR => MuxReset,
		Q => MuxFfDatOut
		);
--
end  Mux2To1EnaRst_arch ;