------------------------------------------------------------------------------------------------------------------------
-- Copyright (c) 1996-2003 Xilinx, Inc.
-- All Rights Reserved
------------------------------------------------------------------------------------------------------------------------
--
--  Disclaimer:
--              LIMITED WARRANTY AND DISCLAIMER.
--              These designs are provided to you "as is". Xilinx and its licensors make and you
--              receive no warranties or conditions, express, implied, statutory or otherwise, and
--              Xilinx specifically disclaims any implied warranties of merchantability, non-infringement,
--              or fitness for a particular purpose. Xilinx does not warrant that the functions contained
--              in these designs will meet your requirements, or that the operation of these designs will
--              be uninterrupted or error free, or that defects in the designs will be corrected.
--              Furthermore, Xilinx does not warrant or make any representations regarding use or the
--              results of the use of the designs in terms of correctness, accuracy, reliability, or otherwise.
--
--              LIMITATION OF LIABILITY.
--              In no event will Xilinx or its licensors be liable for any loss of data, lost profits, cost or
--              procurement of substitute goods or services, or for any special, incidental, consequential,
--              or indirect damages arising from the use or operation of the designs or accompanying
--              documentation, however caused and on any theory of liability. This limitation will apply
--              even if Xilinx has been advised of the possibility of such damage. This limitation shall
--              apply not-withstanding the failure of the essential purpose of any limited remedies herein.
--
--  Contact:    e-mail  hotline@xilinx.com        phone   + 1 800 255 7778
--   ____  ____
--  /   /\/   /
-- /___/  \  /   Vendor: Xilinx
-- \   \   \/    Version: 0.01
--  \   \        Filename: OclockSerdes.vhd
--  /   /        Date Last Modified:  
-- /___/   /\    Date Created: 24/08/05
-- \   \  /  \
--  \___\/\___\
-- 
-- Device: Virtex-4
-- Author: Marc Defossez
-- Entity Name:  OclockSerdes
-- Purpose: 4-bit output serdes
-- Tools: ISE7.1.03
-- Limitations: none
--
-- Revision History:
--    Rev. 
--
------------------------------------------------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_UNSIGNED.all;
-- Simulation libraries
	-- synopsys translate_off
	library UNISIM;
	use UNISIM.all;
	-- synopsys translate_on
-- end simulation libraries

-- Libraries for Synplicity
-- Needed for components as LUT, SRL which get an INIT parameter.
-- When this library is not mentioned Synplify_Pro fails during synthesis of
-- the hardware with an error about the INIT value.
-- library virtex2;
-- use virtex2.components.all;
-- This library is ignored by Mentor Precision.
-- Only a warning is generated.
-- End libraries for Synplicity

entity OclockSerdes is
    port (
    	-- System
        ClkSrdsClk		: in std_logic;		-- High speed clock from DCM = TxClk90
        ClkSrdsRst		: in std_logic;		-- System Reset
        ClkSrdsMuxSel	: in std_logic; 	-- System Output Enable
        ClkSrdsDataIn	: in std_logic_vector (3 downto 0);
        -- Interface
        LxClkOut_p	: out std_logic;	-- DDR FPGA output
        LxClkOut_n	: out std_logic		-- DDR FPGA output
    );
end  OclockSerdes ;

architecture OclockSerdes_arch of OclockSerdes  is
------------------------------------------------------------------------------------------------------------------------
-- Component Instantiation
------------------------------------------------------------------------------------------------------------------------
component OBUFDS_LVDS_25 port (I : in std_logic; O, OB : out std_logic); end component;
component ODDR	generic (DDR_CLK_EDGE : string); 
				port (D1, D2, C, CE, R, S : in std_logic; Q : out std_logic); end component;
component FDC_1	port (D, C, CLR : in std_logic; Q : out std_logic); end component;

component Mux2To1EnaRst
	port (
		MuxClock	: in std_logic;
		MuxReset	: in std_logic;
		MuxDatIn0	: in std_logic;
		MuxDatIn1	: in std_logic;
		MuxLutEna	: in std_logic;
		MuxFfEna	: in std_logic;
		MuxSelect	: in std_logic;
		MuxCbDatOut	: out std_logic;
		MuxFfDatOut	: out std_logic
	);
end component;
------------------------------------------------------------------------------------------------------------------------
-- Signal, Constants and Attributes Declarations
------------------------------------------------------------------------------------------------------------------------
-- Constants
type RlocValueX is array (0 to 1) of integer;
constant RlocVal_X : RlocValueX := (
	1,	-- 
	1
);
type RlocValueY is array (0 to 1) of integer;
constant RlocVal_Y : RlocValueY := (
	15,	-- 
	14
);
-- Signals
signal High	: std_logic;
signal Low	: std_logic;

signal ClkSrds_IntMuxClkToOddr : std_logic_vector (1 downto 0);
signal ClkSrds_IntOddrToObuf : std_logic;
-- Attributes
attribute RLOC : string;
-- attribute DDR_CLK_EDGE : string;
--	attribute DDR_CLK_EDGE of LxClk_Oddr : label is "SAME_EDGE";
------------------------------------------------------------------------------------------------------------------------
--
begin
--------------------------------------------------------------------
-- Stuff
--------------------------------------------------------------------
High <= '1';
Low <= '0';
--------------------------------------------------------------------
-- Inputs and Outputs
--------------------------------------------------------------------
LxClk_Obuf	: OBUFDS_LVDS_25	port map (I => ClkSrds_IntOddrToObuf, O => LxClkOut_p, OB => LxClkOut_n);

LxClk_Oddr : ODDR
generic map (
	DDR_CLK_EDGE => "SAME_EDGE"	-- SAME_EDGE mode used.
	)
port map (
	D1	=> ClkSrds_IntMuxClkToOddr(0),
	D2	=> ClkSrds_IntMuxClkToOddr(1),
	C	=> ClkSrdsClk,				-- Done to get data out a rising edge of clock
	CE	=> High,
	R	=> ClkSrdsRst,
	S	=> Low,
	Q	=> ClkSrds_IntOddrToObuf
);
--------------------------------------------------------------------
-- Multiplexers and Registers
--------------------------------------------------------------------
GenOne : for I in 0 to 1 generate
--	attribute RLOC of LxClk_Mux : label is "X" & integer'image(RlocVal_X(I)) & "Y" & integer'image(RlocVal_Y(I));
	begin
	LxClk_Mux : Mux2To1EnaRst
	port map (
		MuxClock	=> ClkSrdsClk,
		MuxReset	=> ClkSrdsRst,
		MuxDatIn0	=> ClkSrdsDataIn(I),
		MuxDatIn1	=> ClkSrdsDataIn(I+2),
		MuxLutEna	=> High,
		MuxFfEna	=> High,
		MuxSelect	=> ClkSrdsMuxSel,
		MuxCbDatOut	=> open,
		MuxFfDatOut	=> ClkSrds_IntMuxClkToOddr(I)
	);
end generate;
--------------------------------------------------------------------
--
end  OclockSerdes_arch ;