------------------------------------------------------------------------------------------------------------------------
-- Copyright (c) 1996-2003 Xilinx, Inc.
-- All Rights Reserved
------------------------------------------------------------------------------------------------------------------------
--
--  Disclaimer:
--              LIMITED WARRANTY AND DISCLAIMER.
--              These designs are provided to you "as is". Xilinx and its licensors make and you
--              receive no warranties or conditions, express, implied, statutory or otherwise, and
--              Xilinx specifically disclaims any implied warranties of merchantability, non-infringement,
--              or fitness for a particular purpose. Xilinx does not warrant that the functions contained
--              in these designs will meet your requirements, or that the operation of these designs will
--              be uninterrupted or error free, or that defects in the designs will be corrected.
--              Furthermore, Xilinx does not warrant or make any representations regarding use or the
--              results of the use of the designs in terms of correctness, accuracy, reliability, or otherwise.
--
--              LIMITATION OF LIABILITY.
--              In no event will Xilinx or its licensors be liable for any loss of data, lost profits, cost or
--              procurement of substitute goods or services, or for any special, incidental, consequential,
--              or indirect damages arising from the use or operation of the designs or accompanying
--              documentation, however caused and on any theory of liability. This limitation will apply
--              even if Xilinx has been advised of the possibility of such damage. This limitation shall
--              apply not-withstanding the failure of the essential purpose of any limited remedies herein.
--
--  Contact:    e-mail  hotline@xilinx.com        phone   + 1 800 255 7778
--   ____  ____
--  /   /\/   /
-- /___/  \  /   Vendor: Xilinx
-- \   \   \/    Version: 0.01
--  \   \        Filename: OdataSerdes.vhd
--  /   /        Date Last Modified:  
-- /___/   /\    Date Created: 24/08/05
-- \   \  /  \
--  \___\/\___\
-- 
-- Device: Virtex-4
-- Author: Marc Defossez
-- Entity Name:  OdataSerdes
-- Purpose: 4-bit output serdes
-- Tools: ISE7.1.03
-- Limitations: none
--
-- Revision History:
--    Rev. 
--
------------------------------------------------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_UNSIGNED.all;
-- Simulation libraries
	-- synopsys translate_off
	library UNISIM;
	use UNISIM.all;
	-- synopsys translate_on
-- end simulation libraries

-- Libraries for Synplicity
-- Needed for components as LUT, SRL which get an INIT parameter.
-- When this library is not mentioned Synplify_Pro fails during synthesis of
-- the hardware with an error about the INIT value.
-- library virtex2;
-- use virtex2.components.all;
-- This library is ignored by Mentor Precision.
-- Only a warning is generated.
-- End libraries for Synplicity

entity OdataSerdes is
    port (
    	-- System
        DatSrdsClk		: in std_logic;		-- High speed clock from DCM
        DatSrdsDatIn	: in std_logic_vector (15 downto 0);	-- Data from the back-end system
        DatSrdsRst		: in std_logic;		-- System Reset
        DatSrdsMuxSel	: in std_logic;		-- Multiplexer select input
        -- Interface
        LxDato_p	: out std_logic_vector (3 downto 0);	-- DDR FPGA output
        LxDato_n	: out std_logic_vector (3 downto 0)	-- DDR FPGA output
    );
end  OdataSerdes ;

architecture OdataSerdes_arch of OdataSerdes  is
------------------------------------------------------------------------------------------------------------------------
-- Component Instantiation
------------------------------------------------------------------------------------------------------------------------
component Mux2To1EnaRst
	port (
		MuxClock	: in std_logic;
		MuxReset	: in std_logic;
		MuxDatIn0	: in std_logic;
		MuxDatIn1	: in std_logic;
		MuxLutEna	: in std_logic;
		MuxFfEna	: in std_logic;
		MuxSelect	: in std_logic;
		MuxCbDatOut	: out std_logic;
		MuxFfDatOut	: out std_logic
	);
end component;

component OBUFDS_LVDS_25 port (I : in std_logic; O, OB : out std_logic); end component;
component ODDR	generic (DDR_CLK_EDGE : string); 
				port (D1, D2, C, CE, R, S : in std_logic; Q : out std_logic); end component;
component FDCE_1	port (D, C, CE, CLR : in std_logic; Q : out std_logic); end component;
component FDC_1	port (D, C, CLR : in std_logic; Q : out std_logic); end component;
component FDC	port (D, C, CLR : in std_logic; Q : out std_logic); end component;
component FD_1	port (C, D : in std_logic; Q : out std_logic); end component;
------------------------------------------------------------------------------------------------------------------------
-- Signal, Constants and Attributes Declarations
------------------------------------------------------------------------------------------------------------------------
-- Constants
constant DatSrds_DdrClockEdge : string := "SAME_EDGE";

type RlocValueX is array (0 to 7) of integer;
constant RlocVal_X : RlocValueX := (
	1,	-- 
	1, 
	1, 
	1,
	1,	-- 
	1,
	1,
	1
);
type RlocValueY is array (0 to 7) of integer;
constant RlocVal_Y : RlocValueY := (
	5,	-- 
	10, 
	3, 
	12,
	4,	-- 
	11,
	2,
	13
);
-- Signals
signal Low	: std_logic;
signal High	: std_logic;

signal DatSrds_IntOddrToObuf : std_logic_vector (3 downto 0);
signal DatSrds_IntMuxDatToOddr : std_logic_vector (7 downto 0);

-- Attributes
attribute RLOC : string;
-- attribute DDR_CLK_EDGE : string;
------------------------------------------------------------------------------------------------------------------------
--
begin
--------------------------------------------------------------------
-- Stuff
--------------------------------------------------------------------
Low <= '0';
High <= '1';
--------------------------------------------------------------------
-- Inputs and Outputs
--------------------------------------------------------------------
GenOne : for I in 0 to 3 generate
--	attribute DDR_CLK_EDGE of LxDato_Oddr : label is "SAME_EDGE";
--	begin
	LxDato_Oddr : ODDR
	generic map (
		DDR_CLK_EDGE => DatSrds_DdrClockEdge	-- SAME_EDGE mode used.
		)
	port map (
		D1	=> DatSrds_IntMuxDatToOddr(I),
		D2	=> DatSrds_IntMuxDatToOddr(I+4),
		C	=> DatSrdsClk,
		CE	=> High,
		R	=> DatSrdsRst,
		S	=> Low,
		Q	=> DatSrds_IntOddrToObuf(I)
	);
	LxDato_Obuf	: OBUFDS_LVDS_25	port map (I => DatSrds_IntOddrToObuf(I), O => LxDato_p(I), OB => LxDato_n(I));
end generate;
--------------------------------------------------------------------
-- Multiplexers and Registers
--------------------------------------------------------------------
GenTre : for I in 0 to 7 generate
--	attribute RLOC of LxDato_Mux : label is "X" & integer'image(RlocVal_X(I)) & "Y" & integer'image(RlocVal_Y(I));
	begin
	LxDato_Mux : Mux2To1EnaRst			
	port map (
		MuxClock	=> DatSrdsClk,
		MuxReset	=> DatSrdsRst,
		MuxDatIn0	=> DatSrdsDatIn(I),
		MuxDatIn1	=> DatSrdsDatIn(I+8),
		MuxLutEna	=> High,
		MuxFfEna	=> High,
		MuxSelect	=> DatSrdsMuxSel,
		MuxCbDatOut	=> open,
		MuxFfDatOut	=> DatSrds_IntMuxDatToOddr(I)
	);
end generate;

--------------------------------------------------------------------
--
end  OdataSerdes_arch ;