------------------------------------------------------------------------------------------------------------------------
-- Copyright (c) 1996-2003 Xilinx, Inc.
-- All Rights Reserved
------------------------------------------------------------------------------------------------------------------------
--
--  Disclaimer:
--              LIMITED WARRANTY AND DISCLAIMER.
--              These designs are provided to you "as is". Xilinx and its licensors make and you
--              receive no warranties or conditions, express, implied, statutory or otherwise, and
--              Xilinx specifically disclaims any implied warranties of merchantability, non-infringement,
--              or fitness for a particular purpose. Xilinx does not warrant that the functions contained
--              in these designs will meet your requirements, or that the operation of these designs will
--              be uninterrupted or error free, or that defects in the designs will be corrected.
--              Furthermore, Xilinx does not warrant or make any representations regarding use or the
--              results of the use of the designs in terms of correctness, accuracy, reliability, or otherwise.
--
--              LIMITATION OF LIABILITY.
--              In no event will Xilinx or its licensors be liable for any loss of data, lost profits, cost or
--              procurement of substitute goods or services, or for any special, incidental, consequential,
--              or indirect damages arising from the use or operation of the designs or accompanying
--              documentation, however caused and on any theory of liability. This limitation will apply
--              even if Xilinx has been advised of the possibility of such damage. This limitation shall
--              apply not-withstanding the failure of the essential purpose of any limited remedies herein.
--
--  Contact:    e-mail  hotline@xilinx.com        phone   + 1 800 255 7778
--   ____  ____
--  /   /\/   /
-- /___/  \  /   Vendor: Xilinx
-- \   \   \/    Version: 0.01
--  \   \        Filename: TxBramMem.vhd
--  /   /        Date Last Modified:  
-- /___/   /\    Date Created: 07/09/05
-- \   \  /  \
--  \___\/\___\
-- 
-- Device: Virtex-4
-- Author: Marc Defossez
-- Entity Name:  TxBramMem
-- Purpose: BlocRAM set for the LinkPort Transmitter
-- Tools: ISE7.1.04
-- Limitations: none
--
-- Revision History:
--    Rev. 
--
------------------------------------------------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_UNSIGNED.all;
-- Simulation libraries
	-- synopsys translate_off
	library UNISIM;
	use UNISIM.all;
	-- synopsys translate_on
-- end simulation libraries

-- Libraries for Synplicity
-- Needed for components as LUT, SRL which get an INIT parameter.
-- When this library is not mentioned Synplify_Pro fails during synthesis of
-- the hardware with an error about the INIT value.
-- library virtex2;
-- use virtex2.components.all;
-- This library is ignored by Mentor Precision.
-- Only a warning is generated.
-- End libraries for Synplicity

entity TxBramMem is
    port (
    	RamPortaEna		: in std_logic;
		SysRst			: in std_logic;
		TxClk			: in std_logic;
		RamRdAddr		: in std_logic_vector (11 downto 0);
		RamDataOut		: out std_logic_vector (15 downto 0);
		SysTxData		: in std_logic_vector (127 downto 0);
		SysTxFifoEna	: in std_logic;
		SysClk			: in std_logic;
		RamWrAddr		: in std_logic_vector (8 downto 0)
    );
end  TxBramMem ;

architecture TxBramMem_arch of TxBramMem  is
------------------------------------------------------------------------------------------------------------------------
-- Component Instantiation
------------------------------------------------------------------------------------------------------------------------
component RAMB16_S4_S36
port (												-- Block ram configured as
	DIA    		: in std_logic_vector(3 downto 0);	-- 512 x 32 write, 4096 x 4 read
	ENA    		: in std_logic;
	WEA    		: in std_logic;
	SSRA   		: in std_logic;
	CLKA   		: in std_logic;
	ADDRA  		: in std_logic_vector(11 downto 0);
	DOA    		: out std_logic_vector(3 downto 0);
	DIB    		: in std_logic_vector(31 downto 0);
	DIPB    	: in std_logic_vector(3 downto 0);
	ENB    		: in std_logic;
	WEB    		: in std_logic;
	SSRB   		: in std_logic;
	CLKB   		: in std_logic;
	ADDRB  		: in std_logic_vector(8 downto 0);
	DOB    		: out std_logic_vector(31 downto 0);
	DOPB    	: out std_logic_vector(3 downto 0)
	);
end component;
------------------------------------------------------------------------------------------------------------------------
-- Signal, Constants and Attributes Declarations
------------------------------------------------------------------------------------------------------------------------
-- Constants
type RlocValue is array (0 to 3) of string (4 downto 1);
constant BramRlocVal : RlocValue := (
	"X0Y0", -- BRAM 0
	"X0Y1", -- BRAM 1
	"X0Y2", -- BRAM 2
	"X0Y3"  -- BRAM 3
);
-- Signals
signal Low	: std_logic;
signal HighNible : std_logic_vector (3 downto 0);

-- Attributes
attribute RLOC	: string;
------------------------------------------------------------------------------------------------------------------------
--
begin
--------------------------------------------------------------------
-- Stuff
--------------------------------------------------------------------
Low <= '0';
HighNible <= "1111";
--------------------------------------------------------------------
-- Transmitter Block-RAM 
--------------------------------------------------------------------
GenOne : for N in 0 to 3 generate
	attribute RLOC of TxBram : label is BramRlocVal(N);
	begin
	TxBram : RAMB16_S4_S36
	port map (
		DIA    	=> HighNible,
		ENA    	=> RamPortaEna,
		WEA    	=> Low,
		SSRA   	=> SysRst,
		CLKA   	=> TxClk,
		ADDRA  	=> RamRdAddr,
		DOA    	=> RamDataOut ((3+(n*4)) downto (n*4)),
		DIB    	=> SysTxData ((31+(n*32)) downto (n*32)),
		DIPB    => HighNible,
		ENB    	=> SysTxFifoEna,
		WEB    	=> SysTxFifoEna,
		SSRB   	=> SysRst,
		CLKB   	=> SysClk,
		ADDRB  	=> RamWrAddr,
		DOB    	=> open,
		DOPB    => open
	);
end generate;
--

end  TxBramMem_arch ;