------------------------------------------------------------------------------------------------------------------------
-- Copyright (c) 1996-2003 Xilinx, Inc.
-- All Rights Reserved
------------------------------------------------------------------------------------------------------------------------
--
--  Disclaimer:
--              LIMITED WARRANTY AND DISCLAIMER.
--              These designs are provided to you "as is". Xilinx and its licensors make and you
--              receive no warranties or conditions, express, implied, statutory or otherwise, and
--              Xilinx specifically disclaims any implied warranties of merchantability, non-infringement,
--              or fitness for a particular purpose. Xilinx does not warrant that the functions contained
--              in these designs will meet your requirements, or that the operation of these designs will
--              be uninterrupted or error free, or that defects in the designs will be corrected.
--              Furthermore, Xilinx does not warrant or make any representations regarding use or the
--              results of the use of the designs in terms of correctness, accuracy, reliability, or otherwise.
--
--              LIMITATION OF LIABILITY.
--              In no event will Xilinx or its licensors be liable for any loss of data, lost profits, cost or
--              procurement of substitute goods or services, or for any special, incidental, consequential,
--              or indirect damages arising from the use or operation of the designs or accompanying
--              documentation, however caused and on any theory of liability. This limitation will apply
--              even if Xilinx has been advised of the possibility of such damage. This limitation shall
--              apply not-withstanding the failure of the essential purpose of any limited remedies herein.
--
--  Contact:    e-mail  hotline@xilinx.com        phone   + 1 800 255 7778
--   ____  ____
--  /   /\/   /
-- /___/  \  /   Vendor: Xilinx
-- \   \   \/    Version: 
--  \   \        Filename: ToplevelLinkPort.vhd
--  /   /        Date Last Modified:  
-- /___/   /\    Date Created: 30/08/05
-- \   \  /  \
--  \___\/\___\
-- 
-- Device: Virtex-4
-- Author: Marc Defossez
-- Entity Name:  ToplevelLinkPort
-- Purpose: Toplevel design of a set LinkPort receiver/transmitter.
-- Tools: ISE7.1.03
-- Limitations: none
--
-- Revision History:
--    Rev. 
--
------------------------------------------------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_UNSIGNED.all;
-- Simulation libraries
	-- synopsys translate_off
	library UNISIM;
	use UNISIM.all;
	-- synopsys translate_on
-- end simulation libraries

-- Libraries for Synplicity
-- Needed for components as LUT, SRL which get an INIT parameter.
-- When this library is not mentioned Synplify_Pro fails during synthesis of
-- the hardware with an error about the INIT value.
-- library virtex2;
-- use virtex2.components.all;
-- This library is ignored by Mentor Precision.
-- Only a warning is generated.
-- End libraries for Synplicity

entity ToplevelLinkPort is
    port (
    	-- System
        TopSysClk		: in std_logic;
        TopSysRefClk	: in std_logic;	-- 200 MHz clock for IDELAYCTRL elements.
        TopTxRxClk_p	: in std_logic;
        TopTxRxClk_n	: in std_logic;
        TopSysRst		: in std_logic;		-- Active low RESET
        -- Test Interface
        TopSysStrt		: in std_logic;
        TopSysStop		: in std_logic;
        TopSysFlag		: out std_logic_vector (2 downto 0);
        TopSysRxFifoEna : in std_logic;
        TopSysRxData	: out std_logic_vector (127 downto 0);
        -- Receiver
        LxClkIn_p	: in std_logic;
    	LxClkIn_n	: in std_logic;
    	LxDati_p	: in std_logic_vector (3 downto 0);
    	LxDati_n	: in std_logic_vector (3 downto 0);
    	LxAcko		: out std_logic;
    	LxBcmpiN	: in std_logic;
        --Transmitter
        LxClkOut_p	: out std_logic;
        LxClkOut_n	: out std_logic;
        LxDato_p	: out std_logic_vector (3 downto 0);
        LxDato_n	: out std_logic_vector (3 downto 0);
        LxAcki		: in std_logic;
        LxBcmpoN	: out std_logic
    );
end  ToplevelLinkPort ;

architecture ToplevelLinkPort_arch of ToplevelLinkPort  is
------------------------------------------------------------------------------------------------------------------------
-- Component Instantiation
------------------------------------------------------------------------------------------------------------------------
component LinkPortRx
    port (
    	-- System
    	RxClk		: in std_logic;	-- Clock at same rate of LxClk LinkPort clock.
    	RxRefClk	: in std_logic;	-- Clock (200 MHz) for the IDELAYCTRL element of the receiver.
    	SysClk		: in std_logic;	-- System clock reading the FIFO's.
    	SysRxFifoEna	: in std_logic;	-- System control of FIFO.
    	SysRxEna	: in std_logic; -- Control on the receiver
    	SysRst		: in std_logic;	-- System reset, active low.
    	SysRxFlag 	: out std_logic_vector (2 downto 0); -- FIFO flags for the system.
    	SysRxData	: out std_logic_vector (127 downto 0); -- Received data
    	-- Interface
    	LxClkIn_p	: in std_logic;
    	LxClkIn_n	: in std_logic;
    	LxDati_p	: in std_logic_vector (3 downto 0);
    	LxDati_n	: in std_logic_vector (3 downto 0);
    	LxAcko		: out std_logic;
    	LxBcmpiN	: in std_logic
    );
end component;

component LinkPortTx
    port (
    	-- System
        TxClk		: in std_logic;			-- Used in the whole design.
        TxClk90		: in std_logic;			-- Used by the LxClkOut.
        SysClk		: in std_logic;
        SysRst		: in std_logic;
        SysTxFifoEna : in std_logic;		-- Enable of the TX Bram write side, Fill the RAM side.
        SysTxEna	: in std_logic;			-- Enable of the TX Bram read side, tranmitter side.
        SysTxFifoEmpty	: out std_logic;		-- Flag of empty FIFO.
        SysTxFifoFull	: out std_logic;		-- Flag of Full write-side FIFO.
        SysLxAcki	: out std_logic;		-- Passing of LxAcki signal.
        SysTxData	: in std_logic_vector (127 downto 0);
        -- Interface
        LxClkOut_p	: out std_logic;
        LxClkOut_n	: out std_logic;
        LxDato_p	: out std_logic_vector (3 downto 0);
        LxDato_n	: out std_logic_vector (3 downto 0);
        LxAcki		: in std_logic;
        LxBcmpoN	: out std_logic
    );
end component;

component DCM_BASE
	generic (
		CLKIN_DIVIDE_BY_2		: boolean;
		CLKIN_PERIOD			: real;
		CLKOUT_PHASE_SHIFT		: string;
		CLK_FEEDBACK			: string;
		DCM_AUTOCALIBRATION 	: boolean;
		DCM_PERFORMANCE_MODE 	: string;
		DESKEW_ADJUST 			: string;
		DFS_FREQUENCY_MODE 		: string;
		DLL_FREQUENCY_MODE 		: string;
		DUTY_CYCLE_CORRECTION	: boolean;
		FACTORY_JF 				: bit_vector;
		PHASE_SHIFT 			: integer;
		STARTUP_WAIT 			: boolean
	);
	port (
		CLKIN 		: in std_logic;
		CLKFB 		: in std_logic;
		RST 		: in std_logic;
		CLK0 		: out std_logic;
		CLK90 		: out std_logic;
		CLK180 		: out std_logic;
		CLK270 		: out std_logic;
		CLK2X 		: out std_logic;
		CLK2X180 	: out std_logic;
		CLKDV 		: out std_logic;
		CLKFX 		: out std_logic;
		CLKFX180 	: out std_logic;
		LOCKED 		: out std_logic
	);
end component;

component BUFG 				port (I : in std_logic;		O : out std_logic); end component;
component IBUFG_LVCMOS25 	port (I : in std_logic; 	O : out std_logic); end component;
component IBUF_LVCMOS25 	port (I : in std_logic; 	O : out std_logic); end component;
component OBUF_LVCMOS25_S_12 	port (I : in std_logic; 	O : out std_logic); end component;
component IBUFGDS_LVDS_25	port (I, IB : in std_logic; O : out std_logic); end component;
component FDP		port (D, C, PRE : in std_logic;	Q : out std_logic); end component;
------------------------------------------------------------------------------------------------------------------------
-- Signal, Constants and Attributes Declarations
------------------------------------------------------------------------------------------------------------------------
-- Constants

-- Signals
signal Low	: std_logic;
signal High	: std_logic;
signal IntTopSysRst	: std_logic;
signal IntNotTopSysRst	: std_logic;

signal IntTopSysClk	: std_logic;
signal IntSysDcmClk0ToBufg	: std_logic;
signal IntSysDcmClk90ToBufg : std_logic;
signal IntSysDcmLocked	: std_logic;
signal IntNotSysDcmLocked	: std_logic;

signal IntTopTxRxClk : std_logic;
signal IntTxRxDcmClk0ToBufg	: std_logic;
signal IntTxRxDcmClk90ToBufg : std_logic;
signal IntTxRxDcmLocked	: std_logic;
signal IntNotTxRxDcmLocked	: std_logic;

signal IntSysClk	: std_Logic;
signal IntSysClk90	: std_Logic;
signal IntTxRxClk	: std_Logic;
signal IntTxRxClk90	: std_Logic;

signal IntSysRst	: std_logic;
signal IntSysTxFifoEmpty  : std_logic;
signal IntSysTxFifoFull  : std_logic;
signal IntSysLxAcki : std_logic;
signal IntSrlShft : std_logic_vector (15 downto 0);
signal IntSrlLxAcki : std_logic;
signal IntDataVal	: std_logic_vector (127 downto 0);
signal IntWrite	: std_logic;
signal IntPatCnt	: std_logic_vector (3 downto 0);
signal IntTopSysStrt : std_logic;
signal IntTopSysStop : std_logic;
signal IntStartOne	: std_logic;
signal IntStartTwo	: std_logic;
signal IntStartStop	: std_logic;
signal IntTxStart	: std_logic;
signal IntSysRxEna	: std_logic;
signal IntTopSysFlag : std_logic_vector (2 downto 0);
signal IntTopSysFlagOut : std_logic_vector (2 downto 0);
signal IntTopSysRxData : std_logic_vector (127 downto 0);
signal IntTopSysRxDataOut : std_logic_vector (127 downto 0);
signal IntTopSysRxFifoEna : std_logic;

-- Attributes
-- attribute CLKIN_DIVIDE_BY_2 : string;
-- 	attribute CLKIN_DIVIDE_BY_2 	of SysDcm : label is "FALSE";
-- 	attribute CLKIN_DIVIDE_BY_2 	of TxRxDcm : label is "FALSE";
-- attribute CLKIN_PERIOD : string;
-- 	attribute CLKIN_PERIOD			of SysDcm : label is "8.88888";
-- 	attribute CLKIN_PERIOD			of TxRxDcm : label is "2.22222";
-- attribute CLKOUT_PHASE_SHIFT : string;
-- 	attribute CLKOUT_PHASE_SHIFT	of SysDcm : label is "NONE";
-- 	attribute CLKOUT_PHASE_SHIFT	of TxRxDcm : label is "NONE";
-- attribute CLK_FEEDBACK : string;
-- 	attribute CLK_FEEDBACK			of SysDcm : label is "1X";
-- 	attribute CLK_FEEDBACK			of TxRxDcm : label is "1X";
-- attribute DCM_AUTOCALIBRATION : string;
-- 	attribute DCM_AUTOCALIBRATION	of SysDcm : label is "TRUE";
-- 	attribute DCM_AUTOCALIBRATION	of TxRxDcm : label is "TRUE";
-- attribute DCM_PERFORMANCE_MODE : string;
-- 	attribute DCM_PERFORMANCE_MODE	of SysDcm : label is "MAX_SPEED";
-- 	attribute DCM_PERFORMANCE_MODE	of TxRxDcm : label is "MAX_SPEED";
-- attribute DESKEW_ADJUST : string;
-- 	attribute DESKEW_ADJUST			of SysDcm : label is "SYSTEM_SYNCHRONOUS";
-- 	attribute DESKEW_ADJUST			of TxRxDcm : label is "SYSTEM_SYNCHRONOUS";
-- attribute DFS_FREQUENCY_MODE : string;
-- 	attribute DFS_FREQUENCY_MODE	of SysDcm : label is "LOW";
-- 	attribute DFS_FREQUENCY_MODE	of TxRxDcm : label is "HIGH";
-- attribute DLL_FREQUENCY_MODE : string;
-- 	attribute DLL_FREQUENCY_MODE	of SysDcm : label is "LOW";
-- 	attribute DLL_FREQUENCY_MODE	of TxRxDcm : label is "HIGH";
-- attribute DUTY_CYCLE_CORRECTION : string;
-- 	attribute DUTY_CYCLE_CORRECTION	of SysDcm : label is "TRUE";
-- 	attribute DUTY_CYCLE_CORRECTION	of TxRxDcm : label is "TRUE";
-- attribute FACTORY_JF : string;
-- 	attribute FACTORY_JF			of SysDcm : label is "F0F0";
-- 	attribute FACTORY_JF			of TxRxDcm : label is "F0F0";
-- attribute PHASE_SHIFT : string;
-- 	attribute PHASE_SHIFT			of SysDcm : label is "0";
-- 	attribute PHASE_SHIFT			of TxRxDcm : label is "0";
-- attribute STARTUP_WAIT : string;
-- 	attribute STRATUP_WAIT			of SysDcm : label is "FALSE";
-- 	attribute STRATUP_WAIT			of TxRxDcm : label is "FALSE";
	
attribute RLOC : string;
attribute RLOC_ORIGIN : string;
	attribute RLOC_ORIGIN of RxPort : label is "X0Y168";
	attribute RLOC_ORIGIN of TxPort : label is "X0Y136";
------------------------------------------------------------------------------------------------------------------------
--
begin

--------------------------------------------------------------------
-- Stuff
--------------------------------------------------------------------
Low <= '0';
High <= '1';
SysRst_IBUF	: IBUF_LVCMOS25 port map (I => TopSysRst, O => IntTopSysRst);
IntNotTopSysRst <= not IntTopSysRst;
--------------------------------------------------------------------
-- System DCM
--------------------------------------------------------------------
SysClk_IBUFG : IBUFG_LVCMOS25 port map (I => TopSysClk, O => IntTopSysClk);

SysClk_DCM : DCM_BASE
	generic map (
		CLKIN_DIVIDE_BY_2		=> FALSE,
		CLKIN_PERIOD			=> 10.0,
		CLKOUT_PHASE_SHIFT		=> "NONE",
		CLK_FEEDBACK			=> "1X",
		DCM_AUTOCALIBRATION 	=> TRUE,
		DCM_PERFORMANCE_MODE 	=> "MAX_SPEED",
		DESKEW_ADJUST 			=> "SYSTEM_SYNCHRONOUS",
		DFS_FREQUENCY_MODE 		=> "LOW",
		DLL_FREQUENCY_MODE 		=> "LOW",
		DUTY_CYCLE_CORRECTION	=> TRUE,
		FACTORY_JF 				=> X"F0F0",
		PHASE_SHIFT 			=> 0,
		STARTUP_WAIT 			=> FALSE
	)
	port map (
		CLKIN 		=> IntTopSysClk,
		CLKFB 		=> IntSysClk,
		RST 		=> IntNotTopSysRst,
		CLK0 		=> IntSysDcmClk0ToBufg,
		CLK90 		=> IntSysDcmClk90ToBufg,
		CLK180 		=> open,
		CLK270 		=> open,
		CLK2X 		=> open,
		CLK2X180 	=> open,
		CLKDV 		=> open,
		CLKFX 		=> open,
		CLKFX180 	=> open,
		LOCKED 		=> IntSysDcmLocked
	);

SysClk_BUFG_0	: BUFG port map (I => IntSysDcmClk0ToBufg, O => IntSysClk);
SysClk_BUFG_90  : BUFG port map (I => IntSysDcmClk90ToBufg, O => IntSysClk90);
--------------------------------------------------------------------
-- Tx and RX DCM
--------------------------------------------------------------------
IntNotSysDcmLocked <= not IntSysDcmLocked;

TxRxClk_IBUFGDS : IBUFGDS_LVDS_25 port map (I => TopTxRxClk_p, IB => TopTxRxClk_n, O => IntTopTxRxClk);

TxRxClk_DCM : DCM_BASE
	generic map (
		CLKIN_DIVIDE_BY_2		=> FALSE,
		CLKIN_PERIOD			=> 10.0,
		CLKOUT_PHASE_SHIFT		=> "NONE",
		CLK_FEEDBACK			=> "1X",
		DCM_AUTOCALIBRATION 	=> TRUE,
		DCM_PERFORMANCE_MODE 	=> "MAX_SPEED",
		DESKEW_ADJUST 			=> "SYSTEM_SYNCHRONOUS",
		DFS_FREQUENCY_MODE 		=> "LOW",
		DLL_FREQUENCY_MODE 		=> "LOW",
		DUTY_CYCLE_CORRECTION	=> TRUE,
		FACTORY_JF 				=> X"F0F0",
		PHASE_SHIFT 			=> 0,
		STARTUP_WAIT 			=> FALSE
	)
	port map (
		CLKIN 		=> IntTopTxRxClk,
		CLKFB 		=> IntTxRxClk,
		RST 		=> IntNotSysDcmLocked,
		CLK0 		=> IntTxRxDcmClk0ToBufg,
		CLK90 		=> IntTxRxDcmClk90ToBufg,
		CLK180 		=> open,
		CLK270 		=> open,
		CLK2X 		=> open,
		CLK2X180 	=> open,
		CLKDV 		=> open,
		CLKFX 		=> open,
		CLKFX180 	=> open,
		LOCKED 		=> IntTxRxDcmLocked
	);

TxRxClk_BUFG_0	: BUFG port map (I => IntTxRxDcmClk0ToBufg, O => IntTxRxClk);
TxRxClk_BUFG_90  : BUFG port map (I => IntTxRxDcmClk90ToBufg, O => IntTxRXClk90);

IntNotTxRxDcmLocked <= not IntTxRxDcmLocked;
SysRstFf_FDP : FDP
	port map (
		D	=> IntNotTxRxDcmLocked,
		C	=> IntSysClk,
		PRE	=> IntNotTopSysRst,
		Q	=> IntSysRst
	);
--------------------------------------------------------------------
-- Transmitter
--------------------------------------------------------------------
TxPort : LinkPortTx
    port map (
    	-- System
        TxClk		 	=> IntTxRxClk,
        TxClk90		 	=> IntTxRxClk90,
        SysClk		 	=> IntSysClk,
        SysRst		 	=> IntSysRst,
        SysTxFifoEna 	=> IntWrite,
        SysTxEna	 	=> IntTxStart,
        SysTxFifoFull 	=> IntSysTxFifoFull,
        SysTxFifoEmpty	=> IntSysTxFifoEmpty,
        SysLxAcki	 	=> IntSysLxAcki,
        SysTxData	 	=> IntDataVal,
        -- Interface
        LxClkOut_p	 => LxClkOut_p,
        LxClkOut_n	 => LxClkOut_n,
        LxDato_p	 => LxDato_p,
        LxDato_n	 => LxDato_n,
        LxAcki		 => LxAcki,
        LxBcmpoN	 => LxBcmpoN
    );
--------------------------------------------------------------------
-- Receiver
--------------------------------------------------------------------
RxPort : LinkPortRx
    port map (
    	-- System
    	RxClk		 => IntTxRxClk,
    	RxRefClk	 => TopSysRefClk,
    	SysClk		 => IntSysClk,
    	SysRxFifoEna => IntTopSysRxFifoEna,
    	SysRxEna	 => IntSysRxEna,
    	SysRst		 => IntSysRst,
    	SysRxFlag 	 => IntTopSysFlag,
    	SysRxData	 => IntTopSysRxData,
    	-- Interface
    	LxClkIn_p	 => LxClkIn_p,
    	LxClkIn_n	 => LxClkIn_n,
    	LxDati_p	 => LxDati_p,
    	LxDati_n	 => LxDati_n,
    	LxAcko		 => LxAcko,
    	LxBcmpiN	 => LxBcmpiN
    );
--------------------------------------------------------------------
-- Logic
--------------------------------------------------------------------
-- Pattern generator and BRAM fill
--------------------------------------------------------------------
BramWrData : process (IntSysClk, IntSysRst)
begin
if (IntSysRst = '1') then
	IntDataVal <= (others => '0');
	IntWrite <= '0';
	IntPatCnt <= "0000";
elsif (IntSysClk'event and IntSysClk = '1') then
	if (IntSysTxFifoFull = '0' and IntSysTxFifoEmpty = '1') then
		IntPatCnt <= IntPatCnt + 1;
	elsif (IntSysTxFifoFull /= '1') then
		IntPatCnt <= IntPatCnt + 1;
		if (IntPatCnt = "1111") then
			IntDataVal <= ((IntDataVal(127 downto 112) - '1') & (IntDataVal(111 downto 96) + '1') &
			         (IntDataVal(95 downto 80) - '1') & (IntDataVal(79 downto 64) + '1') &
			         (IntDataVal(63 downto 48) - '1') & (IntDataVal(47 downto 32) + '1') &
			         (IntDataVal(31 downto 16) - '1') & (IntDataVal(15 downto 0) + '1'));
			IntWrite <= '1';
		else
			IntWrite <= '0';
		end if ;
	else 
		IntWrite <= '0';
	end if;
end if ;
end process;
--------------------------------------------------------------------
-- Transmission Start
--------------------------------------------------------------------
-- Sample LxAcki and wait for x clock cycles to start the transmitter.
SampleLxAcki : process (IntSysClk, IntSysLxAcki)
begin
if (IntSysClk'event and IntSysClk = '1') then
	IntSrlShft <= IntSysLxAcki & IntSrlShft(15 downto 1);
end if;

end process;
-- Start the transmitter when LxAcki is high and the transmit fifo is full or a start command is given.
TxStart : process (IntSysClk, IntSysRst)
begin
if (IntSysRst = '1') then
	IntTxStart <= '0';
elsif (IntSysClk'event and IntSysClk = '1') then
	if (IntSrlShft(0) = '1' and (IntSysTxFifoFull = '1' or IntStartStop = '1')) then
		IntTxStart <= '1';
	end if;
end if;
end process;
--
--------------------------------------------------------------------
-- Receiver Start
--------------------------------------------------------------------
-- LxAcko must be set high to enable and start the trsnmission.
--
SysStrt_IBUF	: IBUF_LVCMOS25 port map (I => TopSysStrt, O => IntTopSysStrt);
SysStop_IBUF	: IBUF_LVCMOS25 port map (I => TopSysStop, O => IntTopSysStop);
--
StartOne : process (IntSysClk, IntTopSysStrt, IntTopSysStop)
begin
if (IntTopSysStop = '1') then
	IntStartOne <= '0';
elsif (IntSysClk'event and IntSysClk = '1') then
	IntStartOne <= IntTopSysStrt;
end if;
end process;

StartTwo : process (IntSysClk, IntSysRst)
begin
if (IntSysRst = '1') then
	IntStartTwo <= '0';
	IntStartStop <= '0';
	IntSysRxEna <= '1';
elsif (IntSysClk'event and IntSysClk = '1') then
	IntStartTwo <= IntStartOne;
	if (IntStartOne = '1' and IntStartTwo = '1') then
		IntStartStop <= '1';
	else
		IntStartStop <= '0';
	end if;
	IntSysRxEna <= IntStartStop;
end if;
end process;

--
-- Receiver data and Fifo flags
--
SysRxFifoEna_IBUF	: IBUF_LVCMOS25 port map (I => TopSysRxFifoEna, O => IntTopSysRxFifoEna);

TheRxFlags : for I in 2 downto 0 generate
	process (IntSysClk, IntSysRst)
	begin
	if (IntSysRst = '1') then
		IntTopSysFlagOut(I) <= '0';
	elsif (IntSysClk'event and IntSysClk = '1') then
		IntTopSysFlagOut(I) <= IntTopSysFlag(I);
	end if;
	end process;
	
	SysFlags_OBUF	: OBUF_LVCMOS25_S_12 port map (I => IntTopSysFlagOut(I), O => TopSysFlag(I));
end generate;

TheRxData : for I in 127 downto 0 generate
	process (IntSysClk, IntSysRst)
	begin
	if (IntSysRst = '1') then
		IntTopSysRxDataOut(I) <= '0';
	elsif (IntSysClk'event and IntSysClk = '1') then
		IntTopSysRxDataOut(I) <= IntTopSysRxData(I);
	end if;
	end process;
	
	SysFlags_OBUF	: OBUF_LVCMOS25_S_12 port map (I => IntTopSysRxDataOut(I), O => TopSysRxData(I));
end generate;
--
--------------------------------------------------------------------
end  ToplevelLinkPort_arch;